from janito.agent.runtime_config import runtime_config


def handle_prompt(console, shell_state=None, **kwargs):
    profile_manager = kwargs.get("profile_manager")
    prompt = profile_manager.system_prompt_template if profile_manager else None
    if not prompt and profile_manager:
        prompt = profile_manager.system_prompt_template
    console.print(f"[bold magenta]System Prompt:[/bold magenta]\n{prompt}")


handle_prompt.help_text = "Show the system prompt"


def handle_role(console, args=None, shell_state=None, **kwargs):
    profile_manager = (
        shell_state.profile_manager
        if shell_state and hasattr(shell_state, "profile_manager")
        else kwargs.get("profile_manager")
    )
    if not args:
        console.print("[bold red]Usage: /role <new role description>[/bold red]")
        return
    new_role = " ".join(args)
    if profile_manager:
        profile_manager.set_role(new_role)
    # Update system message in conversation
    found = False
    for msg in shell_state.conversation_history.get_messages():
        if msg.get("role") == "system":
            msg["content"] = (
                profile_manager.system_prompt_template if profile_manager else new_role
            )
            found = True
            break
    if not found:
        if shell_state and hasattr(shell_state, "conversation_history"):
            shell_state.conversation_history.set_system_message(new_role)
    # Also store the raw role string
    if profile_manager:
        setattr(profile_manager, "role_name", new_role)
    runtime_config.set("role", new_role)
    console.print(f"[bold green]System role updated to:[/bold green] {new_role}")


handle_role.help_text = "Change the system role"


def handle_profile(console, *args, **kwargs):
    """/profile - Show the current and available Agent Profile (only 'base' is supported)"""
    console.print("[bold green]Current profile:[/bold green] base")
    console.print("[bold yellow]Available profiles:[/bold yellow]\n- base")



from janito.agent.rich_message_handler import RichMessageHandler
from prompt_toolkit.history import InMemoryHistory
from dataclasses import dataclass, field
from typing import Optional, Any, Dict
from janito.shell.prompt.session_setup import (
    setup_prompt_session,
    print_welcome_message,
)
from janito.shell.commands import handle_command
from janito.agent.conversation_exceptions import EmptyResponseError, ProviderError
from janito.agent.api_exceptions import ApiError
from janito.agent.llm_conversation_history import LLMConversationHistory
import janito.i18n as i18n
from janito.agent.runtime_config import runtime_config
from rich.console import Console
import os
from janito.shell.session.manager import get_session_id
from prompt_toolkit.formatted_text import HTML
import time
from janito.shell.input_history import UserInputHistory


@dataclass
class ShellState:
    mem_history: Any = field(default_factory=InMemoryHistory)
    conversation_history: Any = field(default_factory=lambda: LLMConversationHistory())
    last_usage_info: Dict[str, int] = field(
        default_factory=lambda: {
            "prompt_tokens": 0,
            "completion_tokens": 0,
            "total_tokens": 0,
        }
    )
    last_elapsed: Optional[float] = None
    termweb_stdout_path: Optional[str] = None
    termweb_stderr_path: Optional[str] = None
    livereload_stdout_path: Optional[str] = None
    livereload_stderr_path: Optional[str] = None
    paste_mode: bool = False
    profile_manager: Optional[Any] = None
    user_input_history: Optional[Any] = None


# Track the active prompt session for cleanup
active_prompt_session = None


def load_session(shell_state, continue_session, session_id, profile_manager):
    from janito.shell.session.manager import load_conversation_by_session_id

    if continue_session and session_id:
        try:
            messages, prompts, usage = load_conversation_by_session_id(session_id)
        except FileNotFoundError as e:
            shell_state.profile_manager.agent.message_handler.console.print(
                f"[bold red]{str(e)}[/bold red]"
            )
            return False
        shell_state.conversation_history = LLMConversationHistory(messages)
        conversation_history = shell_state.conversation_history
        found = False
        for msg in conversation_history.get_messages():
            if msg.get("role") == "system":
                msg["content"] = profile_manager.system_prompt_template
                found = True
                break
        if not found:
            conversation_history.set_system_message(
                profile_manager.system_prompt_template
            )
        shell_state.last_usage_info = usage or {}
    else:
        conversation_history = shell_state.conversation_history
        if (
            profile_manager.system_prompt_template
            and (
                not runtime_config.get("vanilla_mode", False)
                or runtime_config.get("system_prompt_template")
            )
            and not any(
                m.get("role") == "system" for m in conversation_history.get_messages()
            )
        ):
            conversation_history.set_system_message(
                profile_manager.system_prompt_template
            )
    return True


def handle_prompt_loop(
    shell_state, session, profile_manager, agent, max_rounds, session_id
):
    global active_prompt_session
    conversation_history = shell_state.conversation_history
    message_handler = RichMessageHandler()
    console = message_handler.console
    while True:
        try:
            if shell_state.paste_mode:
                user_input = session.prompt("Multiline> ", multiline=True)
                was_paste_mode = True
                shell_state.paste_mode = False
            else:
                user_input = session.prompt(
                    HTML("<inputline>💬 </inputline>"), multiline=False
                )
                was_paste_mode = False
        except EOFError:
            console.print("\n[bold red]Exiting...[/bold red]")
            break
        except KeyboardInterrupt:
            console.print()
            try:
                confirm = (
                    session.prompt(
                        HTML(
                            "<inputline>Do you really want to exit? (y/n): </inputline>"
                        )
                    )
                    .strip()
                    .lower()
                )
            except KeyboardInterrupt:
                message_handler.handle_message(
                    {"type": "error", "message": "Exiting..."}
                )
                break
            if confirm == "y":
                message_handler.handle_message(
                    {"type": "error", "message": "Exiting..."}
                )
                conversation_history.add_message(
                    {"role": "system", "content": "[Session ended by user]"}
                )
                break
            else:
                continue
        cmd_input = user_input.strip().lower()
        if not was_paste_mode and (cmd_input.startswith("/") or cmd_input == "exit"):
            result = handle_command(
                user_input.strip(),
                console,
                shell_state=shell_state,
            )
            if result == "exit":
                conversation_history.add_message(
                    {"role": "system", "content": "[Session ended by user]"}
                )
                break
            continue
        if not user_input.strip():
            continue
        shell_state.mem_history.append_string(user_input)
        shell_state.user_input_history.append(user_input)
        conversation_history.add_message({"role": "user", "content": user_input})
        handle_chat(shell_state, profile_manager, agent, max_rounds, session_id)
    # Save conversation history after exiting
    save_conversation_history(conversation_history, session_id)


def handle_keyboard_interrupt(conversation_history, message_handler):
    removed_count = 0
    while (
        conversation_history.last_message()
        and conversation_history.last_message().get("role") != "assistant"
    ):
        conversation_history.remove_last_message()
        removed_count += 1
    # Remove the assistant message itself, if present
    if (
        conversation_history.last_message()
        and conversation_history.last_message().get("role") == "assistant"
    ):
        conversation_history.remove_last_message()
        removed_count += 1
    message_handler.handle_message(
        {
            "type": "info",
            "message": f"\nLast turn cleared due to interruption. Returning to prompt. (removed {removed_count} last msgs)\n",
        }
    )


def handle_chat_error(message_handler, error_type, error):
    if error_type == "ProviderError":
        message_handler.handle_message(
            {"type": "error", "message": f"Provider error: {error}"}
        )
    elif error_type == "EmptyResponseError":
        message_handler.handle_message({"type": "error", "message": f"Error: {error}"})
    elif error_type == "ApiError":
        message_handler.handle_message({"type": "error", "message": str(error)})
    elif error_type == "Exception":
        import traceback

        tb = traceback.format_exc()
        message_handler.handle_message(
            {
                "type": "error",
                "message": f"Unexpected error: {error}\n{tb}\nReturning to prompt.",
            }
        )


def handle_chat(shell_state, profile_manager, agent, max_rounds, session_id):
    conversation_history = shell_state.conversation_history
    message_handler = RichMessageHandler()
    console = message_handler.console
    start_time = time.time()
    try:
        response = profile_manager.agent.chat(
            conversation_history,
            max_rounds=max_rounds,
            message_handler=message_handler,
            spinner=True,
        )
    except KeyboardInterrupt:
        handle_keyboard_interrupt(conversation_history, message_handler)
        return
    except ProviderError as e:
        handle_chat_error(message_handler, "ProviderError", e)
        return
    except EmptyResponseError as e:
        handle_chat_error(message_handler, "EmptyResponseError", e)
        return
    except ApiError as e:
        handle_chat_error(message_handler, "ApiError", e)
        return
    except Exception as e:
        handle_chat_error(message_handler, "Exception", e)
        return
    shell_state.last_elapsed = time.time() - start_time
    usage = response.get("usage")
    if usage:
        for k in ("prompt_tokens", "completion_tokens", "total_tokens"):
            shell_state.last_usage_info[k] = usage.get(k, 0)
    content = response.get("content")
    if content and (
        len(conversation_history) == 0
        or conversation_history.get_messages()[-1].get("role") != "assistant"
    ):
        conversation_history.add_message({"role": "assistant", "content": content})


def save_conversation_history(conversation_history, session_id):
    from janito.shell.session.manager import get_session_id

    history_dir = os.path.join(os.path.expanduser("~"), ".janito", "chat_history")
    os.makedirs(history_dir, exist_ok=True)
    session_id_to_save = session_id if session_id else get_session_id()
    history_path = os.path.join(history_dir, f"{session_id_to_save}.json")
    conversation_history.to_json_file(history_path)


def start_chat_shell(
    profile_manager,
    continue_session=False,
    session_id=None,
    max_rounds=100,
    termweb_stdout_path=None,
    termweb_stderr_path=None,
    livereload_stdout_path=None,
    livereload_stderr_path=None,
):
    i18n.set_locale(runtime_config.get("lang", "en"))
    global active_prompt_session
    agent = profile_manager.agent
    message_handler = RichMessageHandler()
    console = message_handler.console
    console.clear()
    shell_state = ShellState()
    shell_state.profile_manager = profile_manager
    user_input_history = UserInputHistory()
    user_input_dicts = user_input_history.load()
    mem_history = shell_state.mem_history
    for item in user_input_dicts:
        if isinstance(item, dict) and "input" in item:
            mem_history.append_string(item["input"])
    shell_state.user_input_history = user_input_history
    if not load_session(shell_state, continue_session, session_id, profile_manager):
        return

    def last_usage_info_ref():
        return shell_state.last_usage_info

    last_elapsed = shell_state.last_elapsed
    print_welcome_message(console, continue_id=session_id if continue_session else None)
    session = setup_prompt_session(
        lambda: shell_state.conversation_history.get_messages(),
        last_usage_info_ref,
        last_elapsed,
        mem_history,
        profile_manager,
        agent,
        lambda: shell_state.conversation_history,
    )
    active_prompt_session = session
    handle_prompt_loop(
        shell_state, session, profile_manager, agent, max_rounds, session_id
    )



import sys
from janito.agent.config import local_config, global_config, CONFIG_OPTIONS
from janito.agent.runtime_config import unified_config, runtime_config
from janito.agent.config_defaults import CONFIG_DEFAULTS
from rich import print
from ._utils import home_shorten
import os
from pathlib import Path


def handle_run_config(args):
    if args.run_config:
        for run_item in args.run_config:
            try:
                key, val = run_item.split("=", 1)
            except ValueError:
                print("Invalid format for --run-config, expected key=val")
                sys.exit(1)
            key = key.strip()
            if key not in CONFIG_OPTIONS:
                print(
                    f"Invalid config key: '{key}'. Supported keys are: {', '.join(CONFIG_OPTIONS.keys())}"
                )
                sys.exit(1)
            runtime_config.set(key, val.strip())
        return True
    return False


def handle_set_local_config(args):
    if args.set_local_config:
        try:
            key, val = args.set_local_config.split("=", 1)
        except ValueError:
            print("Invalid format for --set-local-config, expected key=val")
            sys.exit(1)
        key = key.strip()
        if key not in CONFIG_OPTIONS:
            print(
                f"Invalid config key: '{key}'. Supported keys are: {', '.join(CONFIG_OPTIONS.keys())}"
            )
            sys.exit(1)
        local_config.set(key, val.strip())
        local_config.save()
        runtime_config.set(key, val.strip())
        print(f"Local config updated: {key} = {val.strip()}")
        return True
    return False


def handle_set_global_config(args):
    if args.set_global_config:
        try:
            key, val = args.set_global_config.split("=", 1)
        except ValueError:
            print("Invalid format for --set-global-config, expected key=val")
            sys.exit(1)
        key = key.strip()
        if key not in CONFIG_OPTIONS and not key.startswith("template."):
            print(
                f"Invalid config key: '{key}'. Supported keys are: {', '.join(CONFIG_OPTIONS.keys())}"
            )
            sys.exit(1)
        if key.startswith("template."):
            subkey = key[len("template.") :]
            template_dict = global_config.get("template", {})
            template_dict[subkey] = val.strip()
            global_config.set("template", template_dict)
            global_config.save()
            # Remove legacy flat key if present
            if key in global_config._data:
                del global_config._data[key]
            runtime_config.set("template", template_dict)
            print(f"Global config updated: template.{subkey} = {val.strip()}")
            return True
        else:
            global_config.set(key, val.strip())
            global_config.save()
            runtime_config.set(key, val.strip())
            print(f"Global config updated: {key} = {val.strip()}")
            return True
    return False


def handle_set_api_key(args):
    if args.set_api_key:
        existing = dict(global_config.all())
        existing["api_key"] = args.set_api_key.strip()
        global_config._data = existing
        global_config.save()
        runtime_config.set("api_key", args.set_api_key.strip())
        print("Global API key saved.")
        return True
    return False


def handle_show_config(args):
    if args.show_config:
        local_items = _collect_config_items(local_config, unified_config, True)
        global_items = _collect_config_items(
            global_config, unified_config, False, set(local_items.keys())
        )
        _mask_api_keys(local_items)
        _mask_api_keys(global_items)
        _print_config_items(local_items, global_items)
        _print_default_items(local_items, global_items)
        return True
    return False


def _collect_config_items(config, unified_config, is_local, exclude_keys=None):
    items = {}
    keys = set(config.all().keys())
    if exclude_keys:
        keys = keys - set(exclude_keys)
    for key in sorted(keys):
        if key == "template":
            template_dict = config.get("template", {})
            if template_dict:
                items["template"] = f"({len(template_dict)} keys set)"
                for tkey, tval in template_dict.items():
                    items[f"  template.{tkey}"] = tval
            continue
        if key.startswith("template."):
            continue
        if key == "api_key":
            value = config.get("api_key")
            value = (
                value[:4] + "..." + value[-4:]
                if value and len(value) > 8
                else ("***" if value else None)
            )
        else:
            value = unified_config.get(key)
        items[key] = value
    return items


def _mask_api_keys(cfg):
    if "api_key" in cfg and cfg["api_key"]:
        val = cfg["api_key"]
        cfg["api_key"] = val[:4] + "..." + val[-4:] if len(val) > 8 else "***"


def _print_config_items(local_items, global_items):
    from ._print_config import print_config_items

    print_config_items(local_items, color_label="[cyan]🏠 Local Configuration[/cyan]")
    print_config_items(
        global_items, color_label="[yellow]🌐 Global Configuration[/yellow]"
    )


def _print_default_items(local_items, global_items):
    shown_keys = set(local_items.keys()) | set(global_items.keys())
    default_items = {
        k: v
        for k, v in CONFIG_DEFAULTS.items()
        if k not in shown_keys and k != "api_key"
    }
    if default_items:
        print("[green]🟢 Defaults (not set in config files)[/green]")
        for key, value in default_items.items():
            if key == "system_prompt" and value is None:
                template_path = (
                    Path(__file__).parent
                    / "agent"
                    / "templates"
                    / "system_prompt_template_default.j2"
                )
                print(
                    f"{key} = (default template path: {home_shorten(str(template_path))})"
                )
            else:
                print(f"{key} = {value}")
        print()


def handle_config_reset_local(args):
    if getattr(args, "config_reset_local", False):
        local_path = Path(".janito/config.json")
        if local_path.exists():
            os.remove(local_path)
            print(f"Removed local config file: {local_path}")
        else:
            print(f"Local config file does not exist: {local_path}")
        sys.exit(0)


def handle_config_reset_global(args):
    if getattr(args, "config_reset_global", False):
        global_path = Path.home() / ".janito/config.json"
        if global_path.exists():
            os.remove(global_path)
            print(f"Removed global config file: {global_path}")
        else:
            print(f"Global config file does not exist: {global_path}")
        sys.exit(0)


def handle_config_commands(args):
    did_something = False
    did_something |= handle_run_config(args)
    did_something |= handle_set_local_config(args)
    did_something |= handle_set_global_config(args)
    did_something |= handle_set_api_key(args)
    did_something |= handle_show_config(args)
    handle_config_reset_local(args)
    handle_config_reset_global(args)
    if did_something:
        sys.exit(0)



def handle_livelogs(console, args=None, shell_state=None, **kwargs):
    lines = 20
    if args and len(args) > 0 and str(args[0]).isdigit():
        lines = int(args[0])
    stdout_path = shell_state.termweb_stdout_path if shell_state else None
    stderr_path = shell_state.livereload_stderr_path if shell_state else None
    if not stdout_path and not stderr_path:
        console.print(
            "[yellow][livereload] No livereload log files found for this session.[/yellow]"
        )
        return
    stdout_lines = []
    stderr_lines = []
    if stdout_path:
        try:
            with open(stdout_path, encoding="utf-8") as f:
                stdout_lines = f.readlines()[-lines:]
            if stdout_lines:
                console.print(
                    f"[yellow][livereload][stdout] Tail of {stdout_path}:\n"
                    + "".join(stdout_lines)
                )
        except Exception as e:
            console.print(f"[red][livereload][stdout] Error: {e}[/red]")
    if stderr_path:
        try:
            with open(stderr_path, encoding="utf-8") as f:
                stderr_lines = f.readlines()[-lines:]
            if stderr_lines:
                console.print(
                    f"[red][livereload][stderr] Tail of {stderr_path}:\n"
                    + "".join(stderr_lines)
                )
        except Exception as e:
            console.print(f"[red][livereload][stderr] Error: {e}[/red]")
    if (not stdout_path or not stdout_lines) and (not stderr_path or not stderr_lines):
        console.print("[livereload] No output or errors captured in logs.")


handle_livelogs.help_text = (
    "Show live updates from the server log file (default: server.log)"
)



from enum import Enum, auto
from typing import Any


class EventType(Enum):
    CONTENT = auto()
    INFO = auto()
    SUCCESS = auto()
    ERROR = auto()
    PROGRESS = auto()
    WARNING = auto()
    STDOUT = auto()
    STDERR = auto()
    STREAM = auto()
    STREAM_TOOL_CALL = auto()
    STREAM_END = auto()
    TOOL_CALL = auto()
    TOOL_RESULT = auto()


class Event:
    def __init__(self, type: EventType, payload: Any = None):
        self.type = type
        self.payload = payload



import pytest
import types
import json
from janito.agent import tool_registry
from janito.agent.tool_executor import ToolExecutor


class DummyTool:
    def __call__(self, a, b):
        return a + b


def make_tool_call(name, args, id="dummy-id"):
    # Simulate a tool_call object with .function.name, .function.arguments, and .id
    ToolCall = types.SimpleNamespace
    Function = types.SimpleNamespace
    return ToolCall(function=Function(name=name, arguments=json.dumps(args)), id=id)


def test_handle_tool_call_valid_args():
    # Register dummy tool
    tool_registry._tool_registry["dummy"] = {"function": DummyTool()}
    tool_call = make_tool_call("dummy", {"a": 1, "b": 2})
    tool_entry = tool_registry._tool_registry[tool_call.function.name]
    args = json.loads(tool_call.function.arguments)
    result = ToolExecutor().execute(tool_entry, tool_call, args)
    assert result == 3


def test_handle_tool_call_missing_arg():
    tool_registry._tool_registry["dummy"] = {"function": DummyTool()}
    tool_call = make_tool_call("dummy", {"a": 1})
    with pytest.raises(TypeError) as excinfo:
        tool_entry = tool_registry._tool_registry[tool_call.function.name]
        args = json.loads(tool_call.function.arguments)
        ToolExecutor().execute(tool_entry, tool_call, args)
    assert "missing a required argument" in str(excinfo.value)


def test_handle_tool_call_extra_arg():
    tool_registry._tool_registry["dummy"] = {"function": DummyTool()}
    tool_call = make_tool_call("dummy", {"a": 1, "b": 2, "c": 3})
    with pytest.raises(TypeError) as excinfo:
        tool_entry = tool_registry._tool_registry[tool_call.function.name]
        args = json.loads(tool_call.function.arguments)
        ToolExecutor().execute(tool_entry, tool_call, args)
    assert "got an unexpected keyword argument" in str(excinfo.value)



import os
import shutil
from janito.agent.tool_registry import register_tool
from janito.agent.tools_utils.utils import display_path
from janito.agent.tool_base import ToolBase
from janito.agent.tools_utils.action_type import ActionType
from janito.i18n import tr

from janito.agent.tools.validate_file_syntax.core import validate_file_syntax


@register_tool(name="replace_file")
class ReplaceFileTool(ToolBase):
    """
    Replace the entire content of an existing file. Fails if the file does not exist.
    Args:
        file_path (str): Path to the file to replace content.
        content (str): The full new content to write to the file. You must provide the complete content as it will fully overwrite the existing file—do not use placeholders for original content.
    Returns:
        str: Status message indicating the result. Example:
            - "✅ Successfully replaced the file at ..."

    Note: Syntax validation is automatically performed after this operation.
    """

    def run(self, file_path: str, content: str) -> str:
        from janito.agent.tool_use_tracker import ToolUseTracker

        expanded_file_path = file_path  # Using file_path as is
        disp_path = display_path(expanded_file_path)
        file_path = expanded_file_path
        if not os.path.exists(file_path):
            return tr(
                "❗ Cannot replace: file does not exist at '{disp_path}'.",
                disp_path=disp_path,
            )
        # Check previous operation
        tracker = ToolUseTracker()
        if not tracker.last_operation_is_full_read_or_replace(file_path):
            self.report_info(
                ActionType.WRITE,
                tr("📝 Replace file '{disp_path}' ...", disp_path=disp_path),
            )
            self.report_warning(tr("ℹ️ Missing full view."))
            try:
                with open(file_path, "r", encoding="utf-8", errors="replace") as f:
                    current_content = f.read()
            except Exception as e:
                current_content = f"[Error reading file: {e}]"
            return (
                "⚠️ [missing full view] Update was NOT performed. The full content of the file is included below for your review. Repeat the operation if you wish to proceed.\n"
                f"--- Current content of {disp_path} ---\n"
                f"{current_content}"
            )
        self.report_info(
            ActionType.WRITE,
            tr("📝 Replace file '{disp_path}' ...", disp_path=disp_path),
        )
        backup_path = file_path + ".bak"
        shutil.copy2(file_path, backup_path)
        with open(file_path, "w", encoding="utf-8", errors="replace") as f:
            f.write(content)
        new_lines = content.count("\n") + 1 if content else 0
        self.report_success(tr("✅ {new_lines} lines", new_lines=new_lines))
        msg = tr(
            "✅ Replaced file ({new_lines} lines, backup at {backup_path}).",
            new_lines=new_lines,
            backup_path=backup_path,
        )
        # Perform syntax validation and append result
        validation_result = validate_file_syntax(file_path)
        return msg + f"\n{validation_result}"



from janito.agent.profile_manager import AgentProfileManager
from janito.agent.runtime_config import unified_config, runtime_config
from janito.agent.config import get_api_key


def get_system_prompt_template(args, role):
    system_prompt_template = None
    if getattr(args, "system_prompt_template_file", None):
        with open(args.system_prompt_template_file, "r", encoding="utf-8") as f:
            system_prompt_template = f.read()
        runtime_config.set(
            "system_prompt_template_file", args.system_prompt_template_file
        )
    else:
        system_prompt_template = getattr(
            args, "system_prompt_template", None
        ) or unified_config.get("system_prompt_template")
        if getattr(args, "system_prompt_template", None):
            runtime_config.set("system_prompt_template", system_prompt_template)
        if system_prompt_template is None:
            profile_manager = AgentProfileManager(
                api_key=get_api_key(),
                model=unified_config.get("model"),
                role=role,
                profile_name="base",
                interaction_mode=unified_config.get("interaction_mode", "prompt"),
                verbose_tools=unified_config.get("verbose_tools", False),
                base_url=unified_config.get("base_url", None),
                azure_openai_api_version=unified_config.get(
                    "azure_openai_api_version", "2023-05-15"
                ),
                use_azure_openai=unified_config.get("use_azure_openai", False),
            )
            system_prompt_template = profile_manager.system_prompt_template
    return system_prompt_template



import os

from janito.shell.session.manager import reset_session_id


def handle_restart(console, shell_state=None, **kwargs):
    from janito.shell.session.manager import load_last_conversation, save_conversation

    reset_session_id()
    save_path = os.path.join(".janito", "last_conversation.json")

    # --- Append end-of-conversation message to old history if it exists and is non-trivial ---
    if os.path.exists(save_path):
        try:
            messages, prompts, usage = load_last_conversation(save_path)
            if messages and (
                len(messages) > 1
                or (len(messages) == 1 and messages[0].get("role") != "system")
            ):
                messages.append(
                    {"role": "system", "content": "[Session ended by user]"}
                )
                # Save to permanent chat history (let save_conversation pick session file)
                save_conversation(messages, prompts, usage)
        except Exception as e:
            console.print(
                f"[bold red]Failed to update previous conversation history:[/bold red] {e}"
            )

    # Clear the terminal screen
    console.clear()

    # Reset conversation history using its clear method
    shell_state.conversation_history.clear()

    # Reset tool use tracker
    try:
        from janito.agent.tool_use_tracker import ToolUseTracker

        ToolUseTracker.instance().clear_history()
    except Exception as e:
        console.print(
            f"[bold yellow]Warning: Failed to reset tool use tracker:[/bold yellow] {e}"
        )
    # Set system prompt from agent template if available
    if (
        hasattr(shell_state, "profile_manager")
        and shell_state.profile_manager
        and hasattr(shell_state.profile_manager, "agent")
        and shell_state.profile_manager.agent
        and getattr(shell_state.profile_manager.agent, "system_prompt_template", None)
        and not any(
            m.get("role") == "system"
            for m in shell_state.conversation_history.get_messages()
        )
    ):
        shell_state.conversation_history.set_system_message(
            shell_state.profile_manager.agent.system_prompt_template
        )

    # Reset token usage info in-place so all references (including status bar) are updated
    for k in ("prompt_tokens", "completion_tokens", "total_tokens"):
        if k in shell_state.last_usage_info:
            shell_state.last_usage_info[k] = 0
        else:
            shell_state.last_usage_info[k] = 0
    shell_state.last_elapsed = None

    console.print(
        "[bold green]Conversation history has been started (context reset).[/bold green]"
    )


handle_restart.help_text = "Start a new conversation (reset context)"



from janito.i18n import tr


class QueuedMessageHandler:
    def __init__(self, queue, *args, **kwargs):
        self._queue = queue

    def handle_message(self, msg, msg_type=None):
        # Unified: send content (agent/LLM) messages to the frontend via queue
        if not isinstance(msg, dict):
            raise TypeError(
                tr(
                    "QueuedMessageHandler.handle_message expects a dict with 'type' and 'message', got {msg_type}: {msg!r}",
                    msg_type=type(msg),
                    msg=msg,
                )
            )
        msg_type = msg.get("type", "info")
        # For tool_call and tool_result, print and forward the full dict
        if msg_type in ("tool_call", "tool_result"):
            print(
                tr(
                    "[QueuedMessageHandler] {msg_type}: {msg}",
                    msg_type=msg_type,
                    msg=msg,
                )
            )
            self._queue.put(msg)
            return
        message = msg.get("message", "")
        # For normal agent/user/info messages, emit type 'content' for frontend compatibility
        print(
            tr(
                "[QueuedMessageHandler] {msg_type}: {message}",
                msg_type=msg_type,
                message=message,
            )
        )
        if msg_type == "content":
            self._queue.put({"type": "content", "content": message})
        elif msg_type == "info":
            out = {"type": "info", "message": message}
            if "tool" in msg:
                out["tool"] = msg["tool"]
            self._queue.put(out)
        else:
            out = {"type": msg_type, "message": message}
            if "tool" in msg:
                out["tool"] = msg["tool"]
            self._queue.put(out)



"""
UI helpers for conversation (spinner, verbose output).
"""

from rich.console import Console


def show_spinner(message, func, *args, **kwargs):
    console = Console()
    with console.status(message, spinner="dots") as status:
        result = func(*args, status=status, **kwargs)
        status.stop()
        return result


def print_verbose_event(event):
    print(f"[EVENT] {event}")



import json
from pathlib import Path
from threading import Lock
from .config_defaults import CONFIG_DEFAULTS


class SingletonMeta(type):
    _instances = {}
    _lock: Lock = Lock()

    def __call__(cls, *args, **kwargs):
        with cls._lock:
            if cls not in cls._instances:
                instance = super().__call__(*args, **kwargs)
                cls._instances[cls] = instance
        return cls._instances[cls]


class BaseConfig:
    def __init__(self):
        self._data = {}

    def get(self, key, default=None):
        return self._data.get(key, default)

    def set(self, key, value):
        self._data[key] = value

    def all(self):
        return self._data


class FileConfig(BaseConfig):
    def __init__(self, path):
        super().__init__()
        self.path = Path(path).expanduser()
        self.load()

    def load(self):
        if self.path.exists():
            with open(self.path, "r", encoding="utf-8") as f:
                self._data = json.load(f)
                # Remove keys with value None (null in JSON)
                self._data = {k: v for k, v in self._data.items() if v is not None}

        else:
            self._data = {}

    def save(self):
        self.path.parent.mkdir(parents=True, exist_ok=True)
        with open(self.path, "w", encoding="utf-8") as f:
            json.dump(self._data, f, indent=2)


CONFIG_OPTIONS = {
    "api_key": "API key for OpenAI-compatible service (required)",
    "trust": "Trust mode: suppress all console output (bool, default: False)",
    "model": "Model name to use (e.g., 'gpt-4.1')",
    "base_url": "API base URL (OpenAI-compatible endpoint)",
    "role": "Role description for the Agent Profile (e.g., 'software engineer')",
    "system_prompt_template": "Override the entire Agent Profile prompt text",
    "temperature": "Sampling temperature (float, e.g., 0.0 - 2.0)",
    "max_tokens": "Maximum tokens for model response (int)",
    "use_azure_openai": "Whether to use Azure OpenAI client (default: False)",
    # Accept template.* keys as valid config keys (for CLI validation, etc.)
    "template": "Template context dictionary for Agent Profile prompt rendering (nested)",
    "profile": "Agent Profile name (only 'base' is supported)",
    # Note: template.* keys are validated dynamically, not statically here
}


class BaseConfig:
    def __init__(self):
        self._data = {}

    def get(self, key, default=None):
        return self._data.get(key, default)

    def set(self, key, value):
        self._data[key] = value

    def all(self):
        return self._data

        """
        Returns a dictionary suitable for passing as Jinja2 template variables.
        Merges the nested 'template' dict (if present) and all flat 'template.*' keys.
        Flat keys override nested dict keys if there is a conflict.
        """
        template_vars = dict(self._data.get("template", {}))
        for k, v in self._data.items():
            if k.startswith("template.") and k != "template":
                template_vars[k[9:]] = v
        return template_vars


# Import defaults for reference


class EffectiveConfig:
    """Read-only merged view of local and global configs"""

    def __init__(self, local_cfg, global_cfg):
        self.local_cfg = local_cfg
        self.global_cfg = global_cfg

    def get(self, key, default=None):
        for cfg in (self.local_cfg, self.global_cfg):
            val = cfg.get(key)
            if val is not None:
                # Treat explicit None/null as not set
                if val is None:
                    continue
                return val
        # Use centralized defaults if no config found
        if default is None and key in CONFIG_DEFAULTS:
            return CONFIG_DEFAULTS[key]
        return default

    def all(self):
        merged = {}
        # Start with global, override with local
        for cfg in (self.global_cfg, self.local_cfg):
            merged.update(cfg.all())
        return merged


# Singleton instances

local_config = FileConfig(Path(".janito/config.json"))
global_config = FileConfig(Path.home() / ".janito/config.json")

effective_config = EffectiveConfig(local_config, global_config)


def get_api_key():
    """Retrieve API key from config files (local, then global)."""
    api_key = effective_config.get("api_key")
    if api_key:
        return api_key
    raise ValueError("API key not found. Please configure 'api_key' in your config.")



import os
import tempfile
import shutil
import pytest
from janito.agent.tools_utils.gitignore_utils import GitignoreFilter


def test_gitignore_filter_basic(tmp_path):
    # Create a .gitignore file
    gitignore_content = """
ignored_file.txt
ignored_dir/
*.log
"""
    gitignore_path = tmp_path / ".gitignore"
    gitignore_path.write_text(gitignore_content)

    # Create files and directories
    (tmp_path / "ignored_file.txt").write_text("should be ignored")
    (tmp_path / "not_ignored.txt").write_text("should not be ignored")
    (tmp_path / "ignored_dir").mkdir()
    (tmp_path / "ignored_dir" / "file.txt").write_text("should be ignored")
    (tmp_path / "not_ignored_dir").mkdir()
    (tmp_path / "not_ignored_dir" / "file.txt").write_text("should not be ignored")
    (tmp_path / "file.log").write_text("should be ignored")

    gi = GitignoreFilter(str(gitignore_path))

    assert gi.is_ignored(str(tmp_path / "ignored_file.txt"))
    assert not gi.is_ignored(str(tmp_path / "not_ignored.txt"))
    # Directory itself is not ignored, only its contents
    assert not gi.is_ignored(str(tmp_path / "ignored_dir"))
    assert gi.is_ignored(str(tmp_path / "ignored_dir" / "file.txt"))
    assert not gi.is_ignored(str(tmp_path / "not_ignored_dir"))
    assert not gi.is_ignored(str(tmp_path / "not_ignored_dir" / "file.txt"))
    assert gi.is_ignored(str(tmp_path / "file.log"))

    # Test filter_ignored
    dirs = ["ignored_dir", "not_ignored_dir"]
    files = ["ignored_file.txt", "not_ignored.txt", "file.log"]
    filtered_dirs, filtered_files = gi.filter_ignored(str(tmp_path), dirs, files)
    assert "ignored_dir" not in filtered_dirs
    assert "not_ignored_dir" in filtered_dirs
    assert "ignored_file.txt" not in filtered_files
    assert "file.log" not in filtered_files
    assert "not_ignored.txt" in filtered_files



"""
Tests for OpenAISchemaGenerator class-based API.
"""

import pytest
from janito.agent.openai_schema_generator import OpenAISchemaGenerator


class DummyTool:
    """
    Dummy tool for testing.

    Args:
        foo (str): Foo parameter.
        bar (int): Bar parameter.
    Returns:
        The result as a string.
    """

    name = "dummy_tool"

    def run(self, foo: str, bar: int) -> str:
        """Run the dummy tool."""
        return f"{foo}-{bar}"


# Simulate decorator metadata for tests
DummyTool._tool_run_method = DummyTool().run
DummyTool._tool_name = DummyTool.name


def test_generate_schema_success():
    generator = OpenAISchemaGenerator()
    tool = DummyTool
    schema = generator.generate_schema(tool)
    assert schema["name"] == tool.name
    assert "foo" in schema["parameters"]["properties"]
    assert "bar" in schema["parameters"]["properties"]
    assert schema["parameters"]["properties"]["foo"]["type"] == "string"
    assert schema["parameters"]["properties"]["bar"]["type"] == "integer"
    assert schema["description"].startswith("Dummy tool for testing.")


def test_generate_schema_missing_type():
    class BadTool:
        """
        Args:
            foo (str): Foo parameter.
        Returns:
            String result.
        """

        name = "bad_tool"

        def run(self, foo):
            return str(foo)

    BadTool._tool_run_method = BadTool().run
    BadTool._tool_name = BadTool.name
    generator = OpenAISchemaGenerator()
    with pytest.raises(ValueError):
        generator.generate_schema(BadTool)


def test_generate_schema_missing_doc():
    class BadTool2:
        """
        Args:
            foo (str): Foo parameter.
        Returns:
            String result.
        """

        name = "bad_tool2"

        def run(self, foo: str, bar: int) -> str:
            return str(foo)

    BadTool2._tool_run_method = BadTool2().run
    BadTool2._tool_name = BadTool2.name
    generator = OpenAISchemaGenerator()
    with pytest.raises(ValueError):
        generator.generate_schema(BadTool2)


def test_generate_schema_requires_metadata():
    class NotRegisteredTool:
        def run(self, foo: str) -> str:
            return foo

    generator = OpenAISchemaGenerator()
    with pytest.raises(ValueError):
        generator.generate_schema(NotRegisteredTool)



import tempfile
import os
from janito.agent.tools.python_file_runner import PythonFileRunnerTool


def test_python_file_runner_basic():
    tool = PythonFileRunnerTool()
    with tempfile.NamedTemporaryFile("w", suffix=".py", delete=False) as f:
        f.write('print("file runner test")')
        file_path = f.name
    try:
        result = tool.run(file_path)
        assert "file runner test" in result
        assert "Return code: 0" in result
    finally:
        os.remove(file_path)


def test_python_file_runner_error():
    tool = PythonFileRunnerTool()
    with tempfile.NamedTemporaryFile("w", suffix=".py", delete=False) as f:
        f.write('raise Exception("file error")')
        file_path = f.name
    try:
        result = tool.run(file_path)
        assert "Exception" in result or "file error" in result
        assert "Return code:" in result
    finally:
        os.remove(file_path)



import os
import shutil
import tempfile
from janito.agent.tools.find_files import FindFilesTool


def setup_test_dir():
    base = tempfile.mkdtemp()
    os.makedirs(os.path.join(base, "dirA"))
    os.makedirs(os.path.join(base, "dirB"))
    with open(os.path.join(base, "file1.txt"), "w") as f:
        f.write("abc")
    with open(os.path.join(base, "dirA", "file2.txt"), "w") as f:
        f.write("def")
    with open(os.path.join(base, "dirB", "file2.txt"), "w") as f:
        f.write("ghi")
    return base


def teardown_test_dir(base):
    shutil.rmtree(base)


def test_find_files_returns_files_and_dirs():
    base = setup_test_dir()
    try:
        tool = FindFilesTool()
        # Should match file1.txt, dirA, dirB (dirs), and file2.txt in both subdirs
        result = tool.run(base, "file2.txt dirA dirB", max_depth=2)
        results = set(result.strip().split("\n"))
        expected = {
            os.path.join(base, "dirA"),
            os.path.join(base, "dirB"),
            os.path.join(base, "dirA", "file2.txt"),
            os.path.join(base, "dirB", "file2.txt"),
        }
        assert expected.issubset(results)
    finally:
        teardown_test_dir(base)


def test_find_files_explicit_dir_pattern():
    base = setup_test_dir()
    try:
        tool = FindFilesTool()
        # Only directories, with trailing slash
        result = tool.run(base, "*/", max_depth=1)
        results = set(result.strip().split("\n"))
        expected = {
            os.path.join(base, "dirA") + os.sep,
            os.path.join(base, "dirB") + os.sep,
        }
        assert expected == results
    finally:
        teardown_test_dir(base)



def print_termweb_logs(stdout_path, stderr_path, console):
    try:
        with open(stdout_path, encoding="utf-8") as f:
            stdout_content = f.read().strip()
    except Exception:
        stdout_content = None
    try:
        with open(stderr_path, encoding="utf-8") as f:
            stderr_content = f.read().strip()
    except Exception:
        stderr_content = None
    if stdout_content:
        console.print("[yellow][termweb][stdout] Output:\n" + stdout_content)
    if stderr_content:
        console.print("[red][termweb][stderr] Errors:\n" + stderr_content)
    if not stdout_content and not stderr_content:
        console.print("[termweb] No output or errors captured in logs.")



import sys
import subprocess
import tempfile
import time
import http.client
import os
from rich.console import Console


def wait_for_livereload(port, timeout=3.0):
    deadline = time.time() + timeout
    while time.time() < deadline:
        try:
            conn = http.client.HTTPConnection("localhost", port, timeout=0.5)
            conn.request("GET", "/")
            resp = conn.getresponse()
            if resp.status in (200, 404):
                return True
        except Exception:
            pass
        time.sleep(0.1)
    return False


def start_livereload(selected_port):
    console = Console()
    with console.status("[cyan]Starting live reload server...", spinner="dots"):
        app_py_path = os.path.join(
            os.path.dirname(__file__), "..", "livereload", "app.py"
        )
        app_py_path = os.path.abspath(app_py_path)
        if not os.path.isfile(app_py_path):
            console.print("[red]Could not find livereload app.py![/red]")
            return None, False, None, None
        livereload_stdout = tempfile.NamedTemporaryFile(
            delete=False, mode="w+", encoding="utf-8"
        )
        livereload_stderr = tempfile.NamedTemporaryFile(
            delete=False, mode="w+", encoding="utf-8"
        )
        livereload_proc = subprocess.Popen(
            [sys.executable, app_py_path, "--port", str(selected_port)],
            stdout=livereload_stdout,
            stderr=livereload_stderr,
        )
        if wait_for_livereload(selected_port, timeout=3.0):
            console.print(
                f"LiveReload started... Available at http://localhost:{selected_port}"
            )
            return livereload_proc, True, livereload_stdout.name, livereload_stderr.name
        else:
            livereload_proc.terminate()
            livereload_proc.wait()
            from janito.cli._livereload_log_utils import print_livereload_logs

            console.print(
                f"[red]Failed to start LiveReload on port {selected_port}. Check logs for details.[/red]"
            )
            print_livereload_logs(livereload_stdout.name, livereload_stderr.name)
            return None, False, livereload_stdout.name, livereload_stderr.name



from janito.agent.tools.get_file_outline.python_outline import parse_python_outline


def test_no_overlap_top_level_functions_classes():
    code = [
        "def foo():",  # 1
        "    pass",  # 2
        "def bar():",  # 3
        "    pass",  # 4
        "class Baz:",  # 5
        "    def method(self):",  # 6
        "        pass",  # 7
        "def qux():",  # 8
        "    pass",  # 9
    ]
    outline = parse_python_outline(code)
    # Filter only top-level functions and classes
    tops = [
        x for x in outline if x["type"] in ("function", "class") and not x["parent"]
    ]
    # Sort by start
    tops.sort(key=lambda x: x["start"])
    for i in range(len(tops) - 1):
        assert (
            tops[i]["end"] < tops[i + 1]["start"]
        ), f"Overlap: {tops[i]} and {tops[i+1]}"
    print("test_no_overlap_top_level_functions_classes passed.")


if __name__ == "__main__":
    test_no_overlap_top_level_functions_classes()



from janito.agent.tool_base import ToolBase
from janito.agent.tools_utils.action_type import ActionType
from janito.agent.tool_registry import register_tool
from janito.i18n import tr
import subprocess
import tempfile
import sys
import os


@register_tool(name="run_bash_command")
class RunBashCommandTool(ToolBase):
    """
    Execute a non-interactive command using the bash shell and capture live output.
    This tool explicitly invokes the 'bash' shell (not just the system default shell), so it requires bash to be installed and available in the system PATH. On Windows, this will only work if bash is available (e.g., via WSL, Git Bash, or similar).
    Args:
        command (str): The bash command to execute.
        timeout (int, optional): Timeout in seconds for the command. Defaults to 60.
        require_confirmation (bool, optional): If True, require user confirmation before running. Defaults to False.
        requires_user_input (bool, optional): If True, warns that the command may require user input and might hang. Defaults to False. Non-interactive commands are preferred for automation and reliability.
    Returns:
        str: File paths and line counts for stdout and stderr.
    """

    def run(
        self,
        command: str,
        timeout: int = 60,
        require_confirmation: bool = False,
        requires_user_input: bool = False,
    ) -> str:
        if not command.strip():
            self.report_warning(tr("\u2139\ufe0f Empty command provided."))
            return tr("Warning: Empty command provided. Operation skipped.")
        self.report_info(
            ActionType.EXECUTE,
            tr("🖥️ Run bash command: {command} ...\n", command=command),
        )
        if requires_user_input:
            self.report_warning(
                tr(
                    "\u26a0\ufe0f  Warning: This command might be interactive, require user input, and might hang."
                )
            )
            sys.stdout.flush()
        try:
            with (
                tempfile.NamedTemporaryFile(
                    mode="w+", prefix="run_bash_stdout_", delete=False, encoding="utf-8"
                ) as stdout_file,
                tempfile.NamedTemporaryFile(
                    mode="w+", prefix="run_bash_stderr_", delete=False, encoding="utf-8"
                ) as stderr_file,
            ):
                env = os.environ.copy()
                env["PYTHONIOENCODING"] = "utf-8"
                env["LC_ALL"] = "C.UTF-8"
                env["LANG"] = "C.UTF-8"
                process = subprocess.Popen(
                    ["bash", "-c", command],
                    stdout=subprocess.PIPE,
                    stderr=subprocess.PIPE,
                    text=True,
                    encoding="utf-8",
                    bufsize=1,
                    env=env,
                )
                try:
                    stdout_content, stderr_content = process.communicate(
                        timeout=timeout
                    )
                except subprocess.TimeoutExpired:
                    process.kill()
                    self.report_error(
                        tr(
                            " \u274c Timed out after {timeout} seconds.",
                            timeout=timeout,
                        )
                    )
                    return tr(
                        "Command timed out after {timeout} seconds.", timeout=timeout
                    )
                self.report_success(
                    tr(
                        " \u2705 return code {return_code}",
                        return_code=process.returncode,
                    )
                )
                warning_msg = ""
                if requires_user_input:
                    warning_msg = tr(
                        "\u26a0\ufe0f  Warning: This command might be interactive, require user input, and might hang.\n"
                    )
                max_lines = 100
                stdout_lines = stdout_content.count("\n")
                stderr_lines = stderr_content.count("\n")
                if stdout_lines <= max_lines and stderr_lines <= max_lines:
                    result = warning_msg + tr(
                        "Return code: {return_code}\n--- STDOUT ---\n{stdout_content}",
                        return_code=process.returncode,
                        stdout_content=stdout_content,
                    )
                    if stderr_content.strip():
                        result += tr(
                            "\n--- STDERR ---\n{stderr_content}",
                            stderr_content=stderr_content,
                        )
                    return result
                else:
                    result = warning_msg + tr(
                        "[LARGE OUTPUT]\nstdout_file: {stdout_file} (lines: {stdout_lines})\n",
                        stdout_file=stdout_file.name,
                        stdout_lines=stdout_lines,
                    )
                    if stderr_lines > 0:
                        result += tr(
                            "stderr_file: {stderr_file} (lines: {stderr_lines})\n",
                            stderr_file=stderr_file.name,
                            stderr_lines=stderr_lines,
                        )
                    result += tr(
                        "returncode: {return_code}\nUse the get_lines tool to inspect the contents of these files when needed.",
                        return_code=process.returncode,
                    )
                    return result
        except Exception as e:
            self.report_error(tr(" \u274c Error: {error}", error=e))
            return tr("Error running command: {error}", error=e)



from typing import List
from janito.agent.tool_base import ToolBase
from janito.agent.tools_utils.action_type import ActionType
from janito.agent.tool_registry import register_tool
from janito.i18n import tr
import questionary
from questionary import Style

custom_style = Style(
    [
        ("pointer", "fg:#ffffff bg:#1976d2 bold"),
        ("highlighted", "fg:#ffffff bg:#1565c0 bold"),
        ("answer", "fg:#1976d2 bold"),
        ("qmark", "fg:#1976d2 bold"),
    ]
)
HAND_EMOJI = "🖐️"  # 🖐️


@register_tool(name="present_choices")
class PresentChoicesTool(ToolBase):
    """
    Present a list of options to the user and return the selected option(s).

    Args:
        prompt (str): The prompt/question to display.
        choices (List[str]): List of options to present. Use \n in option text for explicit line breaks if needed.
        multi_select (bool): If True, allow multiple selections.
    Returns:
        str: The selected option(s) as a string, or a message if cancelled.
            - For multi_select=True, returns each selection on a new line, each prefixed with '- '.
            - For multi_select=False, returns the selected option as a string.
            - If cancelled, returns 'No selection made.'
    """

    def run(self, prompt: str, choices: List[str], multi_select: bool = False) -> str:
        if not choices:
            return tr("⚠️ No choices provided.")
        self.report_info(
            ActionType.EXECUTE,
            tr(
                "ℹ️ Prompting user: {prompt} (multi_select={multi_select}) ...",
                prompt=prompt,
                multi_select=multi_select,
            ),
        )
        if multi_select:
            result = questionary.checkbox(
                prompt, choices=choices, style=custom_style, pointer=HAND_EMOJI
            ).ask()
            if result is None:
                return tr("No selection made.")
            return (
                "\n".join(f"- {item}" for item in result)
                if isinstance(result, list)
                else f"- {result}"
            )
        else:
            result = questionary.select(
                prompt, choices=choices, style=custom_style, pointer=HAND_EMOJI
            ).ask()
            if result is None:
                return tr("No selection made.")
            return str(result)



import sys
from janito.agent.llm_conversation_history import LLMConversationHistory
import socket
from janito.agent.profile_manager import AgentProfileManager
from janito.agent.runtime_config import unified_config, runtime_config
from janito.agent.config import get_api_key
from janito import __version__
from janito.agent.conversation_exceptions import (
    MaxRoundsExceededError,
    EmptyResponseError,
    ProviderError,
)
from janito.shell.main import start_chat_shell


def is_port_free(port):
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
        return s.connect_ex(("localhost", port)) != 0


def normalize_args(args):
    if getattr(args, "vanilla", False):
        runtime_config.set("vanilla_mode", True)
    if getattr(args, "ntt", False):
        runtime_config.set("no_tools_tracking", True)
    for flag in [
        "verbose_http",
        "verbose_http_raw",
        "verbose_response",
        "verbose_reason",
        "verbose_tools",
        "verbose_events",
        "verbose_messages",
    ]:
        if hasattr(args, flag):
            runtime_config.set(flag, getattr(args, flag, False))
    if getattr(args, "trust_tools", False):
        runtime_config.set("trust_tools", True)
    if getattr(args, "model", None):
        runtime_config.set("model", args.model)
    if getattr(args, "max_tools", None) is not None:
        runtime_config.set("max_tools", args.max_tools)
    if getattr(args, "verbose_reason", False):
        runtime_config.set("verbose_reason", True)
    if getattr(args, "max_tokens", None) is not None:
        runtime_config.set("max_tokens", args.max_tokens)


def setup_profile_manager(args, role, interaction_mode, profile, lang):
    return AgentProfileManager(
        api_key=get_api_key(),
        model=unified_config.get("model"),
        role=role,
        profile_name=profile,
        interaction_mode=interaction_mode,
        verbose_tools=args.verbose_tools,
        base_url=unified_config.get("base_url", ""),
        azure_openai_api_version=unified_config.get(
            "azure_openai_api_version", "2023-05-15"
        ),
        use_azure_openai=unified_config.get("use_azure_openai", False),
        lang=lang,
    )


def handle_termweb(args, interaction_mode):
    termweb_proc = None
    selected_port = None
    termweb_stdout_path = None
    termweb_stderr_path = None
    if (
        not getattr(args, "no_termweb", False)
        and interaction_mode == "chat"
        and not runtime_config.get("vanilla_mode", False)
        and not getattr(args, "input_arg", None)
    ):
        default_port = 8088
        max_port = 8100
        requested_port = args.termweb_port
        if requested_port == default_port:
            for port in range(default_port, max_port + 1):
                if is_port_free(port):
                    selected_port = port
                    break
            if selected_port is None:
                from rich.console import Console

                console = Console()
                console.print(
                    f"[red]No free port found for termweb in range {default_port}-{max_port}.[/red]"
                )
                sys.exit(1)
        else:
            if not is_port_free(requested_port):
                from rich.console import Console

                console = Console()
                console.print(
                    f"[red]Port {requested_port} is not available for termweb.[/red]"
                )
                sys.exit(1)
            selected_port = requested_port
        runtime_config.set("termweb_port", selected_port)
        from janito.cli.termweb_starter import start_termweb

        termweb_proc, started, termweb_stdout_path, termweb_stderr_path = start_termweb(
            selected_port
        )
        if started:
            from janito.agent.config import local_config

            local_config.set("termweb_last_running_port", selected_port)
            local_config.save()
    return termweb_proc, termweb_stdout_path, termweb_stderr_path


def handle_continue_session(args):
    continue_session = False
    session_id = None
    if getattr(args, "input_arg", None) or getattr(args, "continue_session", False):
        _cont = getattr(args, "continue_session", False)
        if _cont:
            continue_session = True
            session_id = getattr(args, "input_arg", None)
            if session_id is None:
                import os
                import glob

                chat_hist_dir = (
                    os.path.join(os.path.expanduser("~"), ".janito", "chat_history")
                    if not os.path.isabs(".janito")
                    else os.path.join(".janito", "chat_history")
                )
                if not os.path.exists(chat_hist_dir):
                    session_id = None
                else:
                    files = glob.glob(os.path.join(chat_hist_dir, "*.json"))
                    if files:
                        latest = max(files, key=os.path.getmtime)
                        session_id = os.path.splitext(os.path.basename(latest))[0]
                    else:
                        session_id = None
        else:
            continue_session = False
            session_id = None
    return continue_session, session_id


def handle_prompt_mode(args, profile_manager):
    prompt = getattr(args, "input_arg", None)
    from rich.console import Console
    from janito.agent.rich_message_handler import RichMessageHandler

    console = Console()
    message_handler = RichMessageHandler()
    messages = []
    system_prompt_override = runtime_config.get("system_prompt_template")
    if system_prompt_override:
        if not runtime_config.get("vanilla_mode", False) or getattr(
            args, "system", None
        ):
            messages.append({"role": "system", "content": system_prompt_override})
    elif profile_manager.system_prompt_template and not runtime_config.get(
        "vanilla_mode", False
    ):
        messages.append(
            {"role": "system", "content": profile_manager.system_prompt_template}
        )
    messages.append({"role": "user", "content": prompt})
    import time

    info_start_time = None
    if getattr(args, "info", False):
        info_start_time = time.time()
    try:
        max_rounds = 100
        result = profile_manager.agent.chat(
            LLMConversationHistory(messages),
            message_handler=message_handler,
            spinner=True,
            max_rounds=max_rounds,
            tool_user=getattr(args, "tool_user", False),
        )
        if (
            getattr(args, "info", False)
            and info_start_time is not None
            and result is not None
        ):
            usage_info = result.get("usage")
            total_tokens = usage_info.get("total_tokens") if usage_info else None
            prompt_tokens = usage_info.get("prompt_tokens") if usage_info else None
            completion_tokens = (
                usage_info.get("completion_tokens") if usage_info else None
            )
            elapsed = time.time() - info_start_time
            console.print(
                f"[bold green]Total tokens:[/] [yellow]{total_tokens}[/yellow] [bold green]| Input:[/] [cyan]{prompt_tokens}[/cyan] [bold green]| Output:[/] [magenta]{completion_tokens}[/magenta] [bold green]| Elapsed:[/] [yellow]{elapsed:.2f}s[/yellow]",
                style="dim",
            )
    except MaxRoundsExceededError:
        console.print("[red]Max conversation rounds exceeded.[/red]")
    except ProviderError as e:
        console.print(f"[red]Provider error:[/red] {e}")
    except EmptyResponseError as e:
        console.print(f"[red]Error:[/red] {e}")


def run_cli(args):
    if args.version:
        print(f"janito version {__version__}")
        sys.exit(0)
    normalize_args(args)
    role = args.role or unified_config.get("role", "software engineer")
    if args.role:
        runtime_config.set("role", args.role)
    interaction_mode = "chat" if not getattr(args, "prompt", None) else "prompt"
    profile = "base"
    lang = getattr(args, "lang", None) or runtime_config.get("lang", "en")
    profile_manager = setup_profile_manager(args, role, interaction_mode, profile, lang)
    profile_manager.refresh_prompt()
    if getattr(args, "show_system", False):
        print(profile_manager.render_prompt())
        sys.exit(0)
    termweb_proc, termweb_stdout_path, termweb_stderr_path = handle_termweb(
        args, interaction_mode
    )
    try:
        continue_session, session_id = handle_continue_session(args)
        if getattr(args, "input_arg", None):
            from janito.cli.one_shot import run_oneshot_mode

            run_oneshot_mode(args, profile_manager, runtime_config)
            return
        import time

        info_start_time = None
        if getattr(args, "info", False):
            info_start_time = time.time()
        usage_info = start_chat_shell(
            profile_manager,
            continue_session=continue_session,
            session_id=session_id,
            termweb_stdout_path=termweb_stdout_path,
            termweb_stderr_path=termweb_stderr_path,
            livereload_stdout_path=None,
            livereload_stderr_path=None,
        )
        if (
            getattr(args, "info", False)
            and usage_info is not None
            and info_start_time is not None
        ):
            elapsed = time.time() - info_start_time
            from rich.console import Console

            total_tokens = usage_info.get("total_tokens")
            console = Console()
            console.print(
                f"[bold green]Total tokens used:[/] [yellow]{total_tokens}[/yellow] [bold green]| Elapsed time:[/] [yellow]{elapsed:.2f}s[/yellow]"
            )
        sys.exit(0)
    except KeyboardInterrupt:
        from rich.console import Console

        console = Console()
        console.print("[yellow]Interrupted by user.[/yellow]")
    finally:
        if termweb_proc:
            termweb_proc.terminate()
            termweb_proc.wait()



"""
Generates OpenAI-compatible function schemas from Python callables and class docstrings.

- Ensures all parameters are documented and type-annotated.
- Integrates return documentation into the schema description.
- Supports Google, NumPy, and relaxed docstring formats.
"""

import inspect
import re
import typing
from collections import OrderedDict

PYTHON_TYPE_TO_JSON = {
    str: "string",
    int: "integer",
    float: "number",
    bool: "boolean",
    list: "array",
    dict: "object",
}


class OpenAISchemaGenerator:
    """
    Generates OpenAI-compatible function schemas from Python callables and class docstrings.

    - Ensures all parameters are documented and type-annotated.
    - Integrates return documentation into the schema description.
    - Supports Google, NumPy, and relaxed docstring formats.
    """

    def __init__(self):
        pass

    def type_to_json_schema(self, annotation):
        if hasattr(annotation, "__origin__"):
            if annotation.__origin__ is list or annotation.__origin__ is typing.List:
                return {
                    "type": "array",
                    "items": self.type_to_json_schema(annotation.__args__[0]),
                }
            if annotation.__origin__ is dict or annotation.__origin__ is typing.Dict:
                return {"type": "object"}
        return {"type": PYTHON_TYPE_TO_JSON.get(annotation, "string")}

    def parse_param_section(self, lines, param_section_headers):
        param_descs = {}
        in_params = False
        for line in lines:
            stripped_line = line.strip()
            if any(
                stripped_line.lower().startswith(h + ":") or stripped_line.lower() == h
                for h in param_section_headers
            ):
                in_params = True
                continue
            if in_params:
                m = re.match(
                    r"([a-zA-Z_][a-zA-Z0-9_]*)\s*(?:\(([^)]+)\))?\s*[:\-]?\s*(.+)",
                    stripped_line,
                )
                if m:
                    param, _, desc = m.groups()
                    param_descs[param] = desc.strip()
                elif stripped_line and stripped_line[0] != "-":
                    if param_descs:
                        last = list(param_descs)[-1]
                        param_descs[last] += " " + stripped_line
            if (
                stripped_line.lower().startswith("returns:")
                or stripped_line.lower() == "returns"
            ):
                break
        return param_descs

    def parse_return_section(self, lines):
        in_returns = False
        return_desc = ""
        for line in lines:
            stripped_line = line.strip()
            if (
                stripped_line.lower().startswith("returns:")
                or stripped_line.lower() == "returns"
            ):
                in_returns = True
                continue
            if in_returns:
                if stripped_line:
                    return_desc += (" " if return_desc else "") + stripped_line
        return return_desc

    def parse_docstring(self, docstring: str):
        """
        Parses a docstring to extract summary, parameter descriptions, and return description.
        Accepts Google, NumPy, and relaxed formats.
        Returns: summary, {param: description}, return_description
        """
        if not docstring:
            return "", {}, ""
        lines = docstring.strip().split("\n")
        summary = lines[0].strip()
        param_section_headers = ("args", "arguments", "params", "parameters")
        param_descs = self.parse_param_section(lines[1:], param_section_headers)
        return_desc = self.parse_return_section(lines[1:])
        return summary, param_descs, return_desc

    def generate_schema(self, tool_class):
        """
        Generates an OpenAI-compatible function schema for a tool class.
        The tool class must have _tool_run_method and _tool_name attributes set by the tool registration decorator.
        Raises ValueError if the return type is not explicitly str or if any parameter is missing a type hint.
        """
        if not hasattr(tool_class, "_tool_run_method") or not hasattr(
            tool_class, "_tool_name"
        ):
            raise ValueError(
                "Tool class must have _tool_run_method and _tool_name attributes (set by @register_tool)."
            )
        func = tool_class._tool_run_method
        tool_name = tool_class._tool_name
        sig = inspect.signature(func)
        # Enforce explicit str return type
        if sig.return_annotation is inspect._empty or sig.return_annotation is not str:
            raise ValueError(
                f"Tool '{tool_name}' must have an explicit return type of 'str'. Found: {sig.return_annotation}"
            )
        # Enforce type hints for all parameters (except self)
        missing_type_hints = [
            name
            for name, param in sig.parameters.items()
            if name != "self" and param.annotation is inspect._empty
        ]
        if missing_type_hints:
            raise ValueError(
                f"Tool '{tool_name}' is missing type hints for parameter(s): {', '.join(missing_type_hints)}.\n"
                f"All parameters must have explicit type hints for schema generation."
            )
        # Only use the class docstring for schema generation
        class_doc = (
            tool_class.__doc__.strip() if tool_class and tool_class.__doc__ else ""
        )
        summary, param_descs, return_desc = self.parse_docstring(class_doc)
        description = summary
        if return_desc:
            description += f"\n\nReturns: {return_desc}"
        # Check that all parameters in the signature have documentation
        undocumented = [
            name
            for name, param in sig.parameters.items()
            if name != "self" and name not in param_descs
        ]
        if undocumented:
            raise ValueError(
                f"Tool '{tool_name}' is missing docstring documentation for parameter(s): {', '.join(undocumented)}.\n"
                f"Parameter documentation must be provided in the Tool class docstring, not the method docstring."
            )
        properties = OrderedDict()
        required = []
        # Inject tool_call_reason as the first required parameter, unless --ntt is set
        from janito.agent.runtime_config import runtime_config

        if not runtime_config.get("no_tools_tracking", False):
            properties["tool_call_reason"] = {
                "type": "string",
                "description": "The reason or context for why this tool is being called. This is required for traceability.",
            }
            required.append("tool_call_reason")
        for name, param in sig.parameters.items():
            if name == "self":
                continue
            annotation = param.annotation
            pdesc = param_descs.get(name, "")
            schema = self.type_to_json_schema(annotation)
            schema["description"] = pdesc
            properties[name] = schema
            if param.default == inspect._empty:
                required.append(name)
        return {
            "name": tool_name,
            "description": description,
            "parameters": {
                "type": "object",
                "properties": properties,
                "required": required,
            },
        }



import os
import json
from datetime import datetime

# --- Session ID generation ---
_current_session_id = None


def generate_session_id():
    # Use seconds since start of year, encode as base36 for shortness
    now = datetime.now()
    start_of_year = datetime(now.year, 1, 1)
    seconds = int((now - start_of_year).total_seconds())
    chars = "0123456789abcdefghijklmnopqrstuvwxyz"
    out = ""
    n = seconds
    while n:
        n, r = divmod(n, 36)
        out = chars[r] + out
    return out or "0"


def reset_session_id():
    global _current_session_id
    _current_session_id = None


def get_session_id():
    global _current_session_id
    if _current_session_id is None:
        _current_session_id = generate_session_id()
    return _current_session_id


def load_last_summary(path=".janito/last_conversation.json"):
    if not os.path.exists(path):
        return None
    with open(path, "r", encoding="utf-8") as f:
        data = json.load(f)
    return data


def load_last_conversation(path=".janito/last_conversation.json"):
    if not os.path.exists(path):
        return [], [], None
    with open(path, "r", encoding="utf-8") as f:
        data = json.load(f)
    messages = data.get("messages", [])
    prompts = data.get("prompts", [])
    usage = data.get("last_usage_info")
    return messages, prompts, usage


def load_conversation_by_session_id(session_id):
    path = os.path.join(".janito", "chat_history", f"{session_id}.json")
    if not os.path.exists(path):
        raise FileNotFoundError(f"Session file not found: {path}")
    with open(path, "r", encoding="utf-8") as f:
        data = json.load(f)
    messages = data.get("messages", [])
    prompts = data.get("prompts", [])
    usage = data.get("last_usage_info")
    return messages, prompts, usage


def save_conversation(messages, prompts, usage_info=None, path=None):
    # Do not save if only one message and it is a system message (noop session)
    if (
        isinstance(messages, list)
        and len(messages) == 1
        and messages[0].get("role") == "system"
    ):
        return

    if path is None:
        session_id = get_session_id()
        path = os.path.join(".janito", "chat_history", f"{session_id}.json")
    os.makedirs(os.path.dirname(path), exist_ok=True)
    data = {"messages": messages, "prompts": prompts, "last_usage_info": usage_info}

    def usage_serializer(obj):
        if hasattr(obj, "to_dict"):
            return obj.to_dict()
        if hasattr(obj, "__dict__"):
            return obj.__dict__
        return str(obj)

    with open(path, "w", encoding="utf-8") as f:
        json.dump(data, f, indent=2, default=usage_serializer)


def last_conversation_exists(path=".janito/last_conversation.json"):
    if not os.path.exists(path):
        return False
    try:
        with open(path, "r", encoding="utf-8") as f:
            data = json.load(f)
        messages = data.get("messages", [])
        return bool(messages)
    except Exception:
        return False



from janito.agent.rich_message_handler import RichMessageHandler
from janito.agent.tools_utils.action_type import ActionType
from rich.console import Console
from io import StringIO


def test_rich_message_handler_action_type():
    # Redirect Rich console output to a string buffer
    buf = StringIO()
    handler = RichMessageHandler()
    handler.console = Console(file=buf, force_terminal=True, color_system="truecolor")

    # Test different action_types (using Enum values)
    test_cases = [
        (ActionType.READ, "cyan"),
        (ActionType.WRITE, "magenta"),
        (ActionType.EXECUTE, "yellow"),
        (None, "cyan"),
    ]
    for action_type, expected_color in test_cases:
        buf.truncate(0)
        buf.seek(0)
        handler.handle_message(
            {
                "type": "info",
                "message": f"Test message for {action_type}",
                "action_type": action_type,
            }
        )
        output = buf.getvalue()
        # Check that the ANSI color code for the expected color is in the output
        assert (
            expected_color in output or "Test message" in output
        )  # fallback if color codes not present


if __name__ == "__main__":
    test_rich_message_handler_action_type()
    print("RichMessageHandler action_type color test passed.")



from janito.agent.tool_base import ToolBase
from janito.agent.tools_utils.action_type import ActionType
from janito.agent.tool_registry import register_tool


@register_tool(name="search_outline")
class SearchOutlineTool(ToolBase):
    """
    Tool for searching outlines in files.
    """

    def run(self, file_path: str) -> str:
        from janito.agent.tools_utils.utils import display_path
        from janito.i18n import tr

        self.report_info(
            ActionType.READ,
            tr(
                "🔍 Searching for outline in '{disp_path}'",
                disp_path=display_path(file_path),
            ),
        )
        # ... rest of implementation ...
        # Example warnings and successes:
        # self.report_warning(tr("No files found with supported extensions."))
        # self.report_warning(tr("Error reading {file_path}: {error}", file_path=file_path, error=e))
        # self.report_success(tr("✅ {count} {match_word} found", count=len(output), match_word=pluralize('match', len(output))))
        pass



import os
from janito.i18n import tr
from janito.agent.tool_base import ToolBase
from janito.agent.tools_utils.action_type import ActionType
from janito.agent.tool_registry import register_tool
from janito.agent.tools_utils.utils import display_path

from .python_validator import validate_python
from .json_validator import validate_json
from .yaml_validator import validate_yaml
from .ps1_validator import validate_ps1
from .xml_validator import validate_xml
from .html_validator import validate_html
from .markdown_validator import validate_markdown
from .js_validator import validate_js
from .css_validator import validate_css


def _get_validator(ext):
    """Return the appropriate validator function for the file extension."""
    mapping = {
        ".py": validate_python,
        ".pyw": validate_python,
        ".json": validate_json,
        ".yml": validate_yaml,
        ".yaml": validate_yaml,
        ".ps1": validate_ps1,
        ".xml": validate_xml,
        ".html": validate_html,
        ".htm": validate_html,
        ".md": validate_markdown,
        ".js": validate_js,
        ".css": validate_css,
    }
    return mapping.get(ext)


def _handle_validation_error(e, report_warning):
    msg = tr("\u26a0\ufe0f Warning: Syntax error: {error}", error=e)
    if report_warning:
        report_warning(msg)
    return msg


def validate_file_syntax(
    file_path: str, report_info=None, report_warning=None, report_success=None
) -> str:
    ext = os.path.splitext(file_path)[1].lower()
    validator = _get_validator(ext)
    try:
        if validator:
            return validator(file_path)
        else:
            msg = tr("\u26a0\ufe0f Warning: Unsupported file extension: {ext}", ext=ext)
            if report_warning:
                report_warning(msg)
            return msg
    except Exception as e:
        return _handle_validation_error(e, report_warning)


@register_tool(name="validate_file_syntax")
class ValidateFileSyntaxTool(ToolBase):
    """
    Validate a file for syntax issues.

    Supported types:
      - Python (.py, .pyw)
      - JSON (.json)
      - YAML (.yml, .yaml)
      - PowerShell (.ps1)
      - XML (.xml)
      - HTML (.html, .htm) [lxml]
      - Markdown (.md)
      - JavaScript (.js)

    Args:
        file_path (str): Path to the file to validate.
    Returns:
        str: Validation status message. Example:
            - "\u2705 Syntax OK"
            - "\u26a0\ufe0f Warning: Syntax error: <error message>"
            - "\u26a0\ufe0f Warning: Unsupported file extension: <ext>"
    """

    def run(self, file_path: str) -> str:
        disp_path = display_path(file_path)
        self.report_info(
            ActionType.READ,
            tr(
                "\U0001f50e Validate syntax for file '{disp_path}' ...",
                disp_path=disp_path,
            ),
        )
        result = validate_file_syntax(
            file_path,
            report_info=self.report_info,
            report_warning=self.report_warning,
            report_success=self.report_success,
        )
        if result.startswith("\u2705"):
            self.report_success(result)
        elif result.startswith("\u26a0\ufe0f"):
            self.report_warning(tr("\u26a0\ufe0f ") + result.lstrip("\u26a0\ufe0f "))
        return result



from typing import List, Dict, Optional
import json
import sys
import traceback


class LLMConversationHistory:
    """
    Manages the message history for a conversation, supporting OpenAI-style roles.
    Intended to be used by ConversationHandler and chat loop for all history operations.
    """

    def __init__(self, messages: Optional[List[Dict]] = None):
        self._messages = messages.copy() if messages else []

    def add_message(self, message: Dict):
        """Append a message dict to the history."""
        content = message.get("content")
        if isinstance(content, str) and any(
            0xD800 <= ord(ch) <= 0xDFFF for ch in content
        ):
            print(
                f"Surrogate code point detected in message content: {content!r}\nStack trace:\n{''.join(traceback.format_stack())}",
                file=sys.stderr,
            )
        self._messages.append(message)

    def get_messages(self, role: Optional[str] = None) -> List[Dict]:
        """
        Return all messages, or only those matching a given role/type (e.g., 'assistant', 'user', 'tool').
        If role is None, returns all messages.
        """
        if role is None:
            return self._messages.copy()
        return [msg for msg in self._messages if msg.get("role") == role]

    def clear(self):
        """Remove all messages from history."""
        self._messages.clear()

    def set_system_message(self, content: str):
        """
        Replace the first system prompt message, or insert if not present.
        """
        system_idx = next(
            (i for i, m in enumerate(self._messages) if m.get("role") == "system"), None
        )
        system_msg = {"role": "system", "content": content}
        if isinstance(content, str) and any(
            0xD800 <= ord(ch) <= 0xDFFF for ch in content
        ):
            print(
                f"Surrogate code point detected in system message content: {content!r}\nStack trace:\n{''.join(traceback.format_stack())}",
                file=sys.stderr,
            )
        if system_idx is not None:
            self._messages[system_idx] = system_msg
        else:
            self._messages.insert(0, system_msg)

    def to_json_file(self, path: str):
        """Save the conversation history as a JSON file to the given path."""
        with open(path, "w", encoding="utf-8") as f:
            json.dump(self.get_messages(), f, indent=2, ensure_ascii=False)

    def __len__(self):
        return len(self._messages)

    def __getitem__(self, idx):
        return self._messages[idx]

    def remove_last_message(self):
        """Remove and return the last message in the history, or None if empty."""
        if self._messages:
            return self._messages.pop()
        return None

    def last_message(self):
        """Return the last message in the history, or None if empty."""
        if self._messages:
            return self._messages[-1]
        return None



from janito.agent.tool_use_tracker import ToolUseTracker
from rich.console import Console
from rich.table import Table


def handle_track(console: Console, args=None, shell_state=None):
    tracker = ToolUseTracker.instance()
    history = tracker.get_history()
    if not history:
        console.print("[bold yellow]No tool usage history found.[/bold yellow]")
        return
    table = Table(show_header=True, header_style="bold magenta")
    table.add_column("#", style="dim", width=4)
    table.add_column("Tool")
    table.add_column("Params/Result")
    for idx, entry in enumerate(history, 1):
        tool = entry["tool"]
        params = entry["params"].copy()
        result = entry.get("result", "")
        # For create/replace file, trim content in params only
        if tool in ("create_file", "replace_file") and "content" in params:
            content = params["content"]
            if isinstance(content, str):
                lines = content.splitlines()
                if len(lines) > 3:
                    params["content"] = (
                        "\n".join(lines[:2])
                        + f"\n... (trimmed, {len(lines)} lines total)"
                    )
                elif len(content) > 120:
                    params["content"] = content[:120] + "... (trimmed)"
                else:
                    params["content"] = content
        param_result = f"{params}\n--- Result ---\n{result}" if result else str(params)
        table.add_row(str(idx), tool, param_result)
    console.print(table)



import os
import tempfile
import pytest
from janito.agent.tools.search_text import SearchTextTool


@pytest.fixture
def sample_text_file():
    fd, path = tempfile.mkstemp(suffix=".txt")
    with os.fdopen(fd, "w", encoding="utf-8") as f:
        f.write("hello world\nthis is a test\nsearch me\nhello again\n")
    yield path
    os.remove(path)


def test_search_text_plain(sample_text_file):
    tool = SearchTextTool()
    result = tool.run(paths=sample_text_file, pattern="hello")
    assert "hello world" in result
    assert "hello again" in result
    assert "this is a test" not in result


def test_search_text_regex(sample_text_file):
    tool = SearchTextTool()
    result = tool.run(paths=sample_text_file, pattern="h.llo", is_regex=True)
    assert "hello world" in result
    assert "hello again" in result
    assert "search me" not in result



def handle_view(console, args=None, shell_state=None, **kwargs):
    messages = shell_state.conversation_history.get_messages()
    if not messages:
        console.print("[yellow]Conversation history is empty.[/yellow]")
        return
    for i, msg in enumerate(messages, 1):
        role = msg.get("role", "?")
        content = msg.get("content", "")
        tool_calls = msg.get("tool_calls")
        tool_call_id = msg.get("tool_call_id")
        console.print(f"[bold]{i}. {role}:[/bold] {content}")
        if tool_calls:
            console.print(f"   [cyan]tool_calls:[/cyan] {tool_calls}")
        if tool_call_id:
            console.print(f"   [magenta]tool_call_id:[/magenta] {tool_call_id}")


handle_view.help_text = "Print the current LLM conversation history"



import os
import tempfile
from janito.agent.tools.validate_file_syntax.core import ValidateFileSyntaxTool

tool = ValidateFileSyntaxTool()


def write_file(path, content):
    with open(path, "w", encoding="utf-8") as f:
        f.write(content)


def test_valid_xml():
    with tempfile.TemporaryDirectory() as tmpdir:
        path = os.path.join(tmpdir, "valid.xml")
        write_file(path, """<root><child>data</child></root>""")
        result = tool.run(path)
        assert "✅ Syntax valid" in result


def test_invalid_xml():
    with tempfile.TemporaryDirectory() as tmpdir:
        path = os.path.join(tmpdir, "invalid.xml")
        write_file(path, """<root><child>data</root>""")
        result = tool.run(path)
        assert "Syntax error" in result


def test_valid_html():
    with tempfile.TemporaryDirectory() as tmpdir:
        path = os.path.join(tmpdir, "valid.html")
        write_file(path, """<html><body><h1>Title</h1></body></html>""")
        result = tool.run(path)
        assert "✅ Syntax valid" in result


def test_invalid_html():
    with tempfile.TemporaryDirectory() as tmpdir:
        path = os.path.join(tmpdir, "invalid.html")
        write_file(path, """<html><body><h1>Title</body></html>""")
        result = tool.run(path)
        assert "Syntax error" in result or "Syntax valid" in result



from janito.agent.tool_registry import get_tool_schemas
from rich.console import Console


def handle_command(cmd, console: Console, shell_state=None):
    cmd = cmd.strip().lower()
    if cmd in ("/exit", "exit"):
        return "exit"
    if cmd in ("/help", "help"):
        console.print("[bold cyan]/help[/]: Show this help message")
        console.print("[bold cyan]/exit[/]: Exit the shell")
        console.print("[bold cyan]/tools[/]: List available tools")
        return
    if cmd in ("/tools", "tools"):
        table = None
        try:
            from rich.table import Table

            table = Table(
                title="Available Tools", show_lines=True, style="bold magenta"
            )
            table.add_column("Name", style="cyan", no_wrap=True)
            table.add_column("Description", style="green")
            table.add_column("Parameters", style="yellow")
            for schema in get_tool_schemas():
                fn = schema["function"]
                params = "\n".join(
                    [
                        f"[bold]{k}[/]: {v['type']}"
                        for k, v in fn["parameters"].get("properties", {}).items()
                    ]
                )
                table.add_row(f"[b]{fn['name']}[/b]", fn["description"], params or "-")
        except Exception as e:
            console.print(f"[red]Error loading tools: {e}[/red]")
        if table:
            console.print(table)
        return
    # Unknown command
    console.print(f"[yellow]Unknown command:[/] {cmd}")



import argparse


def create_parser():
    # Adiciona --list-tools para listar ferramentas registradas
    # (adição incremental segura)

    parser = argparse.ArgumentParser(
        description="OpenRouter API call using OpenAI Python SDK"
    )
    # The positional argument is interpreted as either a prompt or session_id depending on context
    parser.add_argument(
        "input_arg",
        type=str,
        nargs="?",
        help="Prompt to send to the model, or session ID if --continue is used.",
    )

    parser.add_argument(
        "--list",
        nargs="?",
        type=int,
        const=10,
        default=None,
        help="List the last N sessions (default: 10) and exit.",
    )
    parser.add_argument(
        "--view",
        type=str,
        default=None,
        help="View the content of a conversation history by session id and exit.",
    )
    parser.add_argument(
        "--set-provider-config",
        nargs=3,
        metavar=("NAME", "KEY", "VALUE"),
        help="Set a provider config parameter (e.g., --set-provider-config openai api_key sk-xxx).",
    )
    parser.add_argument(
        "--lang",
        type=str,
        default=None,
        help="Language for interface messages (e.g., en, pt). Overrides config if set.",
    )

    parser.add_argument(
        "--app-shell",
        action="store_true",
        help="Use the new prompt_toolkit Application-based chat shell (experimental)",
    )
    parser.add_argument(
        "--max-tokens",
        type=int,
        default=None,
        help="Maximum tokens for model response (overrides config, default: 32000)",
    )
    parser.add_argument(
        "--max-tools",
        type=int,
        default=None,
        help="Maximum number of tool calls allowed within a chat session (default: unlimited)",
    )
    parser.add_argument(
        "--model",
        "-m",
        type=str,
        default=None,
        help="Model name to use for this session (overrides config, does not persist)",
    )
    parser.add_argument(
        "--max-rounds",
        type=int,
        default=None,
        help="Maximum number of agent rounds per prompt (overrides config, default: 50)",
    )

    # Mutually exclusive group for system prompt options
    group = parser.add_mutually_exclusive_group()
    group.add_argument(
        "-s",
        "--system",
        type=str,
        default=None,
        help="Optional system prompt as a raw string.",
    )
    group.add_argument(
        "--system-file",
        type=str,
        default=None,
        help="Path to a plain text file to use as the system prompt (no template rendering, takes precedence over --system-prompt)",
    )

    parser.add_argument(
        "-r",
        "--role",
        type=str,
        default=None,
        help="Role description for the default system prompt",
    )
    parser.add_argument(
        "-t",
        "--temperature",
        type=float,
        default=None,
        help="Sampling temperature (e.g., 0.0 - 2.0)",
    )
    parser.add_argument(
        "--verbose-http", action="store_true", help="Enable verbose HTTP logging"
    )
    parser.add_argument(
        "--verbose-http-raw",
        action="store_true",
        help="Enable raw HTTP wire-level logging",
    )
    parser.add_argument(
        "--verbose-response",
        action="store_true",
        help="Pretty print the full response object",
    )
    parser.add_argument(
        "--list-tools",
        action="store_true",
        help="Lista todas as ferramentas registradas e sai.",
    )
    parser.add_argument(
        "--show-system",
        action="store_true",
        help="Show model, parameters, system prompt, and tool definitions, then exit",
    )
    parser.add_argument(
        "--verbose-reason",
        action="store_true",
        help="Print the tool call reason whenever a tool is invoked (for debugging)",
    )
    parser.add_argument(
        "--verbose-tools",
        action="store_true",
        help="Print tool call parameters and results",
    )
    parser.add_argument(
        "-n",
        "--no-tools",
        action="store_true",
        default=False,
        help="Disable tool use (default: enabled)",
    )
    parser.add_argument(
        "--set-local-config",
        type=str,
        default=None,
        help='Set a local config key-value pair, format "key=val"',
    )
    parser.add_argument(
        "--set-global-config",
        type=str,
        default=None,
        help='Set a global config key-value pair, format "key=val"',
    )
    parser.add_argument(
        "--run-config",
        type=str,
        action="append",
        default=None,
        help='Set a runtime (in-memory only) config key-value pair, format "key=val". Can be repeated.',
    )
    parser.add_argument(
        "--show-config",
        action="store_true",
        help="Show effective configuration and exit",
    )
    parser.add_argument(
        "--set-api-key",
        type=str,
        default=None,
        help="Set and save the API key globally",
    )
    parser.add_argument(
        "--version", action="store_true", help="Show program's version number and exit"
    )
    parser.add_argument(
        "--help-config",
        action="store_true",
        help="Show all configuration options and exit",
    )
    parser.add_argument(
        "--continue-session",
        "--continue",
        action="store_true",
        default=False,
        help="Continue from a saved conversation. Uses the session ID from the positional argument if provided, otherwise resumes the most recent session.",
    )
    parser.add_argument(
        "--web", action="store_true", help="Launch the Janito web server instead of CLI"
    )
    parser.add_argument(
        "--live",
        action="store_true",
        help="Launch the Janito live reload server for web development",
    )
    parser.add_argument(
        "--config-reset-local",
        action="store_true",
        help="Remove the local config file (~/.janito/config.json)",
    )
    parser.add_argument(
        "--config-reset-global",
        action="store_true",
        help="Remove the global config file (~/.janito/config.json)",
    )
    parser.add_argument(
        "--verbose-events",
        action="store_true",
        help="Print all agent events before dispatching to the message handler (for debugging)",
    )
    parser.add_argument(
        "--verbose-messages",
        action="store_true",
        help="Print every new message added to the conversation history with a colored background.",
    )
    parser.add_argument(
        "-V",
        "--vanilla",
        action="store_true",
        default=False,
        help="Vanilla mode: disables tools, system prompt, and temperature (unless -t is set)",
    )
    parser.add_argument(
        "-T",
        "--trust-tools",
        action="store_true",
        help="Suppress all tool output (trusted tools mode: only shows output file locations)",
    )
    parser.add_argument(
        "--profile",
        type=str,
        default=None,
        help="Agent Profile name (only 'base' is supported)",
    )
    parser.add_argument(
        "--no-termweb",
        action="store_true",
        help="Disable the built-in lightweight web file viewer for terminal links (enabled by default)",
    )
    parser.add_argument(
        "--termweb-port",
        type=int,
        default=8088,
        help="Port for the termweb server (default: 8088)",
    )
    parser.add_argument(
        "-i",
        "--info",
        action="store_true",
        help="Show basic program info and exit (useful for one-shot shell execution)",
    )
    parser.add_argument(
        "--ntt",
        action="store_true",
        help="Disable tool call reason tracking (no tools tracking)",
    )
    parser.add_argument(
        "--tool-user",
        action="store_true",
        default=False,
        help="When set, tool responses will use role 'user' instead of 'tool' in the conversation history.",
    )
    return parser



import sys
import os
from livereload import Server


def main():
    port = 35729  # Default livereload port
    if "--port" in sys.argv:
        idx = sys.argv.index("--port")
        if idx + 1 < len(sys.argv):
            try:
                port = int(sys.argv[idx + 1])
            except ValueError:
                pass
    watch_dir = os.path.abspath(os.getcwd())
    server = Server()
    server.watch(watch_dir, delay=1)
    print(
        f"Starting livereload server on http://localhost:{port}, watching {watch_dir}"
    )
    server.serve(root=watch_dir, port=port, open_url_delay=None)


if __name__ == "__main__":
    main()



from janito.agent.conversation_api import (
    get_openai_response,
    retry_api_call,
)
from janito.agent.conversation_tool_calls import handle_tool_calls
from janito.agent.conversation_ui import show_spinner, print_verbose_event
from janito.agent.conversation_exceptions import (
    MaxRoundsExceededError,
    EmptyResponseError,
    NoToolSupportError,
)
from janito.agent.runtime_config import unified_config, runtime_config
from janito.agent.api_exceptions import ApiError
import pprint
from janito.agent.llm_conversation_history import LLMConversationHistory


def get_openai_response_with_content_check(client, model, messages, max_tokens):
    response = get_openai_response(client, model, messages, max_tokens)
    # Check for empty assistant message content, but allow tool/function calls
    if not hasattr(response, "choices") or not response.choices:
        return response  # Let normal error handling occur
    choice = response.choices[0]
    content = getattr(choice.message, "content", None)
    # Check for function_call (legacy OpenAI) or tool_calls (OpenAI v2 and others)
    has_function_call = (
        hasattr(choice.message, "function_call") and choice.message.function_call
    )
    has_tool_calls = hasattr(choice.message, "tool_calls") and choice.message.tool_calls
    if (content is None or str(content).strip() == "") and not (
        has_function_call or has_tool_calls
    ):
        print(
            "[DEBUG] Empty assistant message detected with no tool/function call. Will retry. Raw response:"
        )
        print(repr(response))
        raise EmptyResponseError("Empty assistant message content.")
    return response


class ConversationHandler:
    def __init__(self, client, model):
        self.client = client
        self.model = model
        self.usage_history = []

    @staticmethod
    def remove_system_prompt(messages):
        """
        Return a new messages list with all system prompts removed.
        """
        return [msg for msg in messages if msg.get("role") != "system"]

    def _resolve_max_tokens(self, max_tokens):
        resolved_max_tokens = max_tokens
        if resolved_max_tokens is None:
            resolved_max_tokens = unified_config.get("max_tokens", 32000)
        try:
            resolved_max_tokens = int(resolved_max_tokens)
        except (TypeError, ValueError):
            raise ValueError(
                "max_tokens must be an integer, got: {resolved_max_tokens!r}".format(
                    resolved_max_tokens=resolved_max_tokens
                )
            )
        if runtime_config.get("vanilla_mode", False) and max_tokens is None:
            resolved_max_tokens = 8000
        return resolved_max_tokens

    def _call_openai_api(self, history, resolved_max_tokens, spinner):
        def api_call():
            return get_openai_response_with_content_check(
                self.client,
                self.model,
                history.get_messages(),
                resolved_max_tokens,
            )

        user_message_on_empty = "Received an empty message from you. Please try again."
        if spinner:
            response = show_spinner(
                "Waiting for AI response...",
                retry_api_call,
                api_call,
                history=history,
                user_message_on_empty=user_message_on_empty,
            )
        else:
            response = retry_api_call(
                api_call, history=history, user_message_on_empty=user_message_on_empty
            )
        return response

    def _handle_no_tool_support(self, messages, max_tokens, spinner):
        print(
            "\u26a0\ufe0f Endpoint does not support tool use. Proceeding in vanilla mode (tools disabled)."
        )
        runtime_config.set("vanilla_mode", True)
        resolved_max_tokens = 8000
        if max_tokens is None:
            runtime_config.set("max_tokens", 8000)

        def api_call_vanilla():
            return get_openai_response_with_content_check(
                self.client, self.model, messages, resolved_max_tokens
            )

        user_message_on_empty = "Received an empty message from you. Please try again."
        if spinner:
            response = show_spinner(
                "Waiting for AI response (tools disabled)...",
                retry_api_call,
                api_call_vanilla,
                history=None,
                user_message_on_empty=user_message_on_empty,
            )
        else:
            response = retry_api_call(
                api_call_vanilla,
                history=None,
                user_message_on_empty=user_message_on_empty,
            )
            print(
                "[DEBUG] OpenAI API raw response (tools disabled):",
                repr(response),
            )
        return response, resolved_max_tokens

    def _process_response(self, response):
        if runtime_config.get("verbose_response", False):
            pprint.pprint(response)
        if response is None or not getattr(response, "choices", None):
            error = getattr(response, "error", None)
            if error:
                print(f"ApiError: {error.get('message', error)}")
                raise ApiError(error.get("message", str(error)))
            raise EmptyResponseError(
                f"No choices in response; possible API or LLM error. Raw response: {response!r}"
            )
        choice = response.choices[0]
        usage = getattr(response, "usage", None)
        usage_info = (
            {
                "_debug_raw_usage": getattr(response, "usage", None),
                "prompt_tokens": getattr(usage, "prompt_tokens", None),
                "completion_tokens": getattr(usage, "completion_tokens", None),
                "total_tokens": getattr(usage, "total_tokens", None),
            }
            if usage
            else None
        )
        return choice, usage_info

    def _handle_tool_calls(
        self, choice, history, message_handler, usage_info, tool_user=False
    ):
        tool_responses = handle_tool_calls(
            choice.message.tool_calls, message_handler=message_handler
        )
        agent_idx = len([m for m in history.get_messages() if m.get("role") == "agent"])
        self.usage_history.append({"agent_index": agent_idx, "usage": usage_info})
        history.add_message(
            {
                "role": "assistant",
                "content": choice.message.content,
                "tool_calls": [tc.to_dict() for tc in choice.message.tool_calls],
            }
        )
        for tool_response in tool_responses:
            history.add_message(
                {
                    "role": "user" if tool_user else "tool",
                    "tool_call_id": tool_response["tool_call_id"],
                    "content": tool_response["content"],
                }
            )

    def handle_conversation(
        self,
        messages,
        max_rounds=100,
        message_handler=None,
        verbose_response=False,
        spinner=False,
        max_tokens=None,
        verbose_events=False,
        tool_user=False,
    ):

        if isinstance(messages, LLMConversationHistory):
            history = messages
        else:
            history = LLMConversationHistory(messages)

        if len(history) == 0:
            raise ValueError("No prompt provided in messages")

        resolved_max_tokens = self._resolve_max_tokens(max_tokens)

        for _ in range(max_rounds):
            try:
                response = self._call_openai_api(history, resolved_max_tokens, spinner)
                error = getattr(response, "error", None)
                if error:
                    print(f"ApiError: {error.get('message', error)}")
                    raise ApiError(error.get("message", str(error)))
            except NoToolSupportError:
                response, resolved_max_tokens = self._handle_no_tool_support(
                    messages, max_tokens, spinner
                )
            choice, usage_info = self._process_response(response)
            event = {"type": "content", "message": choice.message.content}
            if runtime_config.get("verbose_events", False):
                print_verbose_event(event)
            if message_handler is not None and choice.message.content:
                message_handler.handle_message(event)
            if not choice.message.tool_calls:
                agent_idx = len(
                    [m for m in history.get_messages() if m.get("role") == "agent"]
                )
                self.usage_history.append(
                    {"agent_index": agent_idx, "usage": usage_info}
                )
                history.add_message(
                    {
                        "role": "assistant",
                        "content": choice.message.content,
                    }
                )
                return {
                    "content": choice.message.content,
                    "usage": usage_info,
                    "usage_history": self.usage_history,
                }
            self._handle_tool_calls(
                choice, history, message_handler, usage_info, tool_user=tool_user
            )
        raise MaxRoundsExceededError(f"Max conversation rounds exceeded ({max_rounds})")



from janito.agent.tool_base import ToolBase
from janito.agent.tool_registry import register_tool
from janito.i18n import tr


@register_tool(name="delete_text_in_file")
class DeleteTextInFileTool(ToolBase):
    """
    Delete all occurrences of text between start_marker and end_marker (inclusive) in a file, using exact string markers.

    Args:
        file_path (str): Path to the file to modify.
        start_marker (str): The starting delimiter string.
        end_marker (str): The ending delimiter string.
        backup (bool, optional): If True, create a backup (.bak) before deleting. Defaults to False.
    Returns:
        str: Status message indicating the result.
    """

    def run(
        self,
        file_path: str,
        start_marker: str,
        end_marker: str,
        backup: bool = False,
    ) -> str:
        import shutil
        from janito.agent.tools_utils.utils import display_path

        disp_path = display_path(file_path)
        backup_path = file_path + ".bak"
        backup_msg = ""
        try:
            with open(file_path, "r", encoding="utf-8", errors="replace") as f:
                content = f.read()
        except Exception as e:
            self.report_error(tr(" ❌ Error reading file: {error}", error=e))
            return tr("Error reading file: {error}", error=e)

        start_count = content.count(start_marker)
        if start_count > 1:
            self.report_error("Need more context for start_marker")
            return (
                f"Error: start_marker is not unique in {disp_path}. "
                "Try including the next line(s) for more context."
            )

        end_count = content.count(end_marker)
        if end_count > 1:
            self.report_error("Need more context for end_marker")
            return (
                f"Error: end_marker is not unique in {disp_path}. "
                "Try including the previous line(s) for more context."
            )

        count = 0
        new_content = content
        while True:
            start_idx = new_content.find(start_marker)
            if start_idx == -1:
                break
            end_idx = new_content.find(end_marker, start_idx + len(start_marker))
            if end_idx == -1:
                break
            # Remove from start_marker to end_marker (inclusive)
            new_content = (
                new_content[:start_idx] + new_content[end_idx + len(end_marker) :]
            )
            count += 1

        if count == 0:
            self.report_warning(tr("ℹ️ No blocks found between markers."))
            return tr(
                "No blocks found between markers in {file_path}.", file_path=file_path
            )

        if backup:
            shutil.copy2(file_path, backup_path)
            backup_msg = f" (A backup was saved to {backup_path})"
        with open(file_path, "w", encoding="utf-8", errors="replace") as f:
            f.write(new_content)

        self.report_success(
            tr(
                "Deleted {count} block(s) between markers in {disp_path}.",
                count=count,
                disp_path=disp_path,
            )
        )
        return (
            tr(
                "Deleted {count} block(s) between markers in {file_path}.",
                count=count,
                file_path=file_path,
            )
            + backup_msg
        )



from janito.agent.conversation_exceptions import (
    MaxRoundsExceededError,
    ProviderError,
    EmptyResponseError,
)
from janito.agent.api_exceptions import ApiError
from janito.agent.llm_conversation_history import LLMConversationHistory


def prepare_messages(args, profile_manager, runtime_config):
    prompt = getattr(args, "input_arg", None)
    messages = []
    system_prompt_override = runtime_config.get("system_prompt_template")
    if system_prompt_override:
        if not runtime_config.get("vanilla_mode", False) or getattr(
            args, "system", None
        ):
            messages.append({"role": "system", "content": system_prompt_override})
    elif profile_manager.system_prompt_template and not runtime_config.get(
        "vanilla_mode", False
    ):
        messages.append(
            {"role": "system", "content": profile_manager.system_prompt_template}
        )
    messages.append({"role": "user", "content": prompt})
    return messages


def print_usage_info(args, info_start_time, result, console):
    if (
        getattr(args, "info", False)
        and info_start_time is not None
        and result is not None
    ):
        usage_info = result.get("usage")
        total_tokens = usage_info.get("total_tokens") if usage_info else None
        prompt_tokens = usage_info.get("prompt_tokens") if usage_info else None
        completion_tokens = usage_info.get("completion_tokens") if usage_info else None
        elapsed = time.time() - info_start_time
        console.print(
            f"[bold green]Total tokens:[/] [yellow]{total_tokens}[/yellow] [bold green]| Input:[/] [cyan]{prompt_tokens}[/cyan] [bold green]| Output:[/] [magenta]{completion_tokens}[/magenta] [bold green]| Elapsed:[/] [yellow]{elapsed:.2f}s[/yellow]",
            style="dim",
        )


def run_oneshot_mode(args, profile_manager, runtime_config):
    from rich.console import Console
    from janito.agent.rich_message_handler import RichMessageHandler
    import time

    console = Console()
    message_handler = RichMessageHandler()
    messages = prepare_messages(args, profile_manager, runtime_config)
    info_start_time = None
    if getattr(args, "info", False):
        info_start_time = time.time()
    try:
        max_rounds = 100
        result = profile_manager.agent.chat(
            LLMConversationHistory(messages),
            message_handler=message_handler,
            spinner=True,
            max_rounds=max_rounds,
        )
        print_usage_info(args, info_start_time, result, console)
    except MaxRoundsExceededError:
        console.print("[red]Max conversation rounds exceeded.[/red]")
    except ProviderError as e:
        console.print(f"[red]Provider error:[/red] {e}")
    except EmptyResponseError as e:
        console.print(f"[red]Error:[/red] {e}")
    except ApiError as e:
        if "maximum context length" in str(e):
            console.print(
                f"[red]Error:[/red] {e}\n[bold yellow]Tip:[/] Try using [green]--max-tokens[/green] with a lower value."
            )
        else:
            console.print(f"[red]API error:[/red] {e}")
    except Exception:
        raise



import sys
import platform
import types
import pytest
from janito.agent.platform_discovery import PlatformDiscovery


def test_platform_name_and_python_version():
    pd = PlatformDiscovery()
    # Platform name should match normalized system
    sys_platform = platform.system().lower()
    expected = (
        "windows"
        if sys_platform.startswith("win")
        else (
            "linux"
            if sys_platform.startswith("linux")
            else "darwin" if sys_platform.startswith("darwin") else sys_platform
        )
    )
    assert pd.get_platform_name() == expected
    # Python version should match
    assert pd.get_python_version() == platform.python_version()


def test_is_windows_linux_mac():
    pd = PlatformDiscovery()
    assert pd.is_windows() == sys.platform.startswith("win")
    assert pd.is_linux() == sys.platform.startswith("linux")
    assert pd.is_mac() == sys.platform.startswith("darwin")


def test_detect_shell_runs_and_returns_str():
    pd = PlatformDiscovery()
    result = pd.detect_shell()
    assert isinstance(result, str)
    assert result  # Should not be empty



class MaxRoundsExceededError(Exception):
    pass


class EmptyResponseError(Exception):
    pass


class ProviderError(Exception):
    def __init__(self, message, error_data):
        self.error_data = error_data
        super().__init__(message)


class NoToolSupportError(Exception):
    """Raised when the API endpoint does not support tool use."""

    pass



from flask import (
    Flask,
    request,
    send_from_directory,
    jsonify,
    render_template,
)
import json
from janito.agent.profile_manager import AgentProfileManager
import os

from janito.agent.runtime_config import unified_config, runtime_config

# Render system prompt from config
role = unified_config.get("role", "software engineer")
system_prompt_template_override = unified_config.get("system_prompt_template")
if system_prompt_template_override:
    system_prompt_template = system_prompt_template_override
else:
    profile_manager = AgentProfileManager(
        api_key=unified_config.get("api_key"),
        model=unified_config.get("model"),
        role=role,
        profile_name="base",
        interaction_mode=unified_config.get("interaction_mode", "prompt"),
        verbose_tools=runtime_config.get("verbose_tools", False),
        base_url=unified_config.get("base_url", None),
        azure_openai_api_version=unified_config.get(
            "azure_openai_api_version", "2023-05-15"
        ),
        use_azure_openai=unified_config.get("use_azure_openai", False),
    )
system_prompt_template = profile_manager.system_prompt_template

app = Flask(
    __name__,
    static_url_path="/static",
    static_folder=os.path.join(os.path.dirname(__file__), "static"),
)

# Secret key for session management
app.secret_key = "replace_with_a_secure_random_secret_key"

# Path for persistent conversation storage
conversation_file = os.path.expanduser("~/.janito/last_conversation_web.json")

# Initially no conversation loaded
conversation = None

# Instantiate the Agent with config-driven parameters (no tool_handler)
agent = profile_manager.agent


@app.route("/get_config")
def get_config():
    # Expose full config for the web app: defaults, effective, runtime (mask api_key)
    from janito.agent.runtime_config import (
        unified_config,
    )  # Kept here: avoids circular import at module level
    from janito.agent.config_defaults import CONFIG_DEFAULTS

    # Start with defaults
    config = dict(CONFIG_DEFAULTS)
    # Overlay effective config
    config.update(unified_config.effective_cfg.all())
    # Overlay runtime config (highest priority)
    config.update(unified_config.runtime_cfg.all())
    api_key = config.get("api_key")
    if api_key:
        config["api_key"] = (
            api_key[:4] + "..." + api_key[-4:] if len(api_key) > 8 else "***"
        )
    return jsonify(config)


@app.route("/set_config", methods=["POST"])
def set_config():
    from janito.agent.runtime_config import runtime_config
    from janito.agent.config import CONFIG_OPTIONS
    from janito.agent.config_defaults import CONFIG_DEFAULTS

    data = request.get_json()
    key = data.get("key")
    value = data.get("value")
    if key not in CONFIG_OPTIONS:
        return (
            jsonify({"status": "error", "message": f"Invalid config key: {key}"}),
            400,
        )
    # Type coercion based on defaults
    default = CONFIG_DEFAULTS.get(key)
    if default is not None and value is not None:
        try:
            if isinstance(default, bool):
                value = bool(value)
            elif isinstance(default, int):
                value = int(value)
            elif isinstance(default, float):
                value = float(value)
            # else: leave as string or None
        except Exception as e:
            return (
                jsonify(
                    {"status": "error", "message": f"Invalid value type for {key}: {e}"}
                ),
                400,
            )
    runtime_config.set(key, value)
    # Mask api_key in response
    resp_value = value
    if key == "api_key" and value:
        resp_value = value[:4] + "..." + value[-4:] if len(value) > 8 else "***"
    return jsonify({"status": "ok", "key": key, "value": resp_value})


@app.route("/favicon.ico")
def favicon():
    return send_from_directory(
        os.path.join(app.root_path, "static"),
        "favicon.ico",
        mimetype="image/vnd.microsoft.icon",
    )


@app.route("/")
def index():
    return render_template("index.html")


@app.route("/load_conversation")
def load_conversation():
    global conversation
    try:
        with open(conversation_file, "r", encoding="utf-8") as f:
            conversation = json.load(f)
    except (FileNotFoundError, json.JSONDecodeError):
        conversation = []
    return jsonify({"status": "ok", "conversation": conversation})


@app.route("/new_conversation", methods=["POST"])
def new_conversation():
    global conversation
    conversation = []
    return jsonify({"status": "ok"})



from janito.agent.message_handler_protocol import MessageHandlerProtocol
from janito.agent.event_handler_protocol import EventHandlerProtocol
from janito.agent.event_dispatcher import EventDispatcher


class DummyMessageHandler(MessageHandlerProtocol):
    def __init__(self):
        self.last_message = None

    def handle_message(self, msg, msg_type=None):
        self.last_message = (msg, msg_type)


class DummyEvent:
    def __init__(self, type_, payload):
        self.type = type_
        self.payload = payload


class DummyEventHandler(EventHandlerProtocol):
    def __init__(self):
        self.last_event = None

    def handle_event(self, event):
        self.last_event = event


def test_message_handler():
    handler = DummyMessageHandler()
    handler.handle_message({"type": "info", "message": "hello"}, "info")
    assert handler.last_message[0]["message"] == "hello"
    print("MessageHandlerProtocol test passed")


def test_event_dispatcher():
    dispatcher = EventDispatcher()
    handler = DummyEventHandler()
    dispatcher.register("test", handler)
    event = DummyEvent("test", {"foo": "bar"})
    dispatcher.dispatch(event)
    assert handler.last_event.payload["foo"] == "bar"
    print("EventHandlerProtocol/EventDispatcher test passed")


if __name__ == "__main__":
    test_message_handler()
    test_event_dispatcher()



# janito/agent/tool_registry.py
from janito.agent.tool_base import ToolBase
from janito.agent.openai_schema_generator import OpenAISchemaGenerator

_tool_registry = {}


def register_tool(tool=None, *, name: str = None):
    if tool is None:
        return lambda t: register_tool(t, name=name)
    override_name = name
    if not (isinstance(tool, type) and issubclass(tool, ToolBase)):
        raise TypeError("Tool must be a class derived from ToolBase.")
    instance = tool()
    if not hasattr(instance, "run") or not callable(instance.run):
        raise TypeError(
            f"Tool '{tool.__name__}' must implement a callable 'call' method."
        )
    tool_name = override_name or instance.name
    # Add metadata for schema generation
    tool._tool_run_method = instance.run
    tool._tool_name = tool_name
    if tool_name in _tool_registry:
        raise ValueError(f"Tool '{tool_name}' is already registered.")
    schema = OpenAISchemaGenerator().generate_schema(tool)
    _tool_registry[tool_name] = {
        "function": instance.run,
        "description": schema["description"],
        "parameters": schema["parameters"],
        "class": tool,
        "instance": instance,
    }
    return tool


def get_tool_schemas():
    schemas = []
    for name, entry in _tool_registry.items():
        schemas.append(
            {
                "type": "function",
                "function": {
                    "name": name,
                    "description": entry["description"],
                    "parameters": entry["parameters"],
                },
            }
        )
    return schemas



from janito.agent.config import local_config, global_config, CONFIG_OPTIONS
from janito.agent.config_defaults import CONFIG_DEFAULTS
from janito.agent.runtime_config import unified_config, runtime_config


def handle_config_shell(console, *args, **kwargs):
    """
    /config show
    /config set local key=value
    /config set global key=value
    /config reset local
    /config reset global
    """
    if not args or args[0] not in ("show", "set", "reset"):
        _print_usage(console)
        return
    if args[0] == "show":
        _show_config(console)
        return
    if args[0] == "reset":
        _reset_config(console, args)
        return
    if args[0] == "set":
        _set_config(console, args)
        return


def _print_usage(console):
    console.print(
        "[bold red]Usage:[/bold red] /config show | /config set local|global key=value | /config reset local|global"
    )


def _show_config(console):
    from janito.cli._print_config import print_full_config

    print_full_config(
        local_config,
        global_config,
        unified_config,
        CONFIG_DEFAULTS,
        console=console,
    )


def _reset_config(console, args):
    if len(args) < 2 or args[1] not in ("local", "global"):
        console.print("[bold red]Usage:[/bold red] /config reset local|global")
        return
    import os
    from pathlib import Path

    scope = args[1]
    if scope == "local":
        local_path = Path(".janito/config.json")
        if local_path.exists():
            os.remove(local_path)
            console.print(f"[green]Removed local config file:[/green] {local_path}")
        else:
            console.print(
                f"[yellow]Local config file does not exist:[/yellow] {local_path}"
            )
    elif scope == "global":
        global_path = Path.home() / ".janito/config.json"
        if global_path.exists():
            os.remove(global_path)
            console.print(f"[green]Removed global config file:[/green] {global_path}")
        else:
            console.print(
                f"[yellow]Global config file does not exist:[/yellow] {global_path}"
            )
    console.print(
        "[bold yellow]Please use /restart for changes to take full effect.[/bold yellow]"
    )


def _set_config(console, args):
    if len(args) < 3 or args[1] not in ("local", "global"):
        console.print("[bold red]Usage:[/bold red] /config set local|global key=value")
        return
    scope = args[1]
    try:
        key, val = args[2].split("=", 1)
    except ValueError:
        console.print("[bold red]Invalid format, expected key=val[/bold red]")
        return
    key = key.strip()
    if key not in CONFIG_OPTIONS and not key.startswith("template."):
        console.print(
            f"[bold red]Invalid config key: '{key}'. Supported keys are: {', '.join(CONFIG_OPTIONS.keys())}"
        )
        return
    val = val.strip()
    if scope == "local":
        local_config.set(key, val)
        local_config.save()
        runtime_config.set(key, val)
        console.print(f"[green]Local config updated:[/green] {key} = {val}")
        console.print(
            "[bold yellow]Please use /restart for changes to take full effect.[/bold yellow]"
        )
    elif scope == "global":
        global_config.set(key, val)
        global_config.save()
        runtime_config.set(key, val)
        console.print(f"[green]Global config updated:[/green] {key} = {val}")
        console.print(
            "[bold yellow]Please use /restart for changes to take full effect.[/bold yellow]"
        )



from janito.agent.tool_registry import get_tool_schemas
from rich.table import Table


def handle_tools(console, args=None, shell_state=None):
    table = Table(title="Available Tools", show_lines=True, style="bold magenta")
    table.add_column("Name", style="cyan", no_wrap=True)
    table.add_column("Description", style="green")
    table.add_column("Parameters", style="yellow")
    try:
        for schema in get_tool_schemas():
            fn = schema["function"]
            params = "\n".join(
                [
                    f"[bold]{k}[/]: {v['type']}"
                    for k, v in fn["parameters"].get("properties", {}).items()
                ]
            )
            table.add_row(f"[b]{fn['name']}[/b]", fn["description"], params or "-")
    except Exception as e:
        console.print(f"[red]Error loading tools: {e}[/red]")
        return
    console.print(table)


handle_tools.help_text = "List available tools"



"""Agent module: defines the core LLM agent with tool and conversation handling."""

import time
from openai import OpenAI
from janito.agent.conversation import ConversationHandler
from janito.agent.conversation_exceptions import ProviderError
from janito.agent.llm_conversation_history import LLMConversationHistory


class Agent:
    """Agent capable of handling conversations and tool calls."""

    REFERER = "www.janito.dev"
    TITLE = "Janito"

    def __init__(
        self,
        api_key: str,
        model: str = None,
        system_prompt_template: str | None = None,
        verbose_tools: bool = False,
        base_url: str = None,
        azure_openai_api_version: str = "2023-05-15",
        use_azure_openai: bool = False,
    ):
        """
        Initialize Agent.

        Args:
            api_key: API key for OpenAI-compatible service.
            model: Model name to use.
            system_prompt_template: Optional system prompt override.
            verbose_tools: Enable verbose tool call logging.
            base_url: API base URL.
            azure_openai_api_version: Azure OpenAI API version (default: "2023-05-15").
            use_azure_openai: Whether to use Azure OpenAI client (default: False).
        """
        self.api_key = api_key
        self.model = model
        self.system_prompt_template = system_prompt_template
        if use_azure_openai:
            # Import inside conditional to avoid requiring AzureOpenAI unless needed
            from openai import AzureOpenAI

            if base_url:
                self.client = AzureOpenAI(
                    api_key=api_key,
                    azure_endpoint=base_url,
                    api_version=azure_openai_api_version,
                )
            else:
                self.client = AzureOpenAI(
                    api_key=api_key,
                    api_version=azure_openai_api_version,
                )
        else:
            if base_url:
                self.client = OpenAI(
                    base_url=base_url,
                    api_key=api_key,
                    default_headers={
                        "HTTP-Referer": self.REFERER,
                        "X-Title": self.TITLE,
                    },
                )
            else:
                self.client = OpenAI(
                    api_key=api_key,
                    default_headers={
                        "HTTP-Referer": self.REFERER,
                        "X-Title": self.TITLE,
                    },
                )

        self.conversation_handler = ConversationHandler(
            self.client,
            self.model,
        )

    @property
    def usage_history(self):
        return self.conversation_handler.usage_history

    def chat(
        self,
        messages=None,
        message_handler=None,
        spinner=False,
        max_tokens=None,
        max_rounds=100,
        tool_user=False,
    ):
        """
        Start a chat conversation with the agent.

        Args:
            messages: LLMConversationHistory instance or None.
            message_handler: Optional handler for event messages.
            spinner: Show spinner during request.
            max_tokens: Max tokens for completion.
            max_rounds: Max conversation rounds.
        Returns:
            dict with 'content', 'usage', and 'usage_history'.
        """
        from janito.agent.runtime_config import runtime_config

        if messages is None:
            messages = LLMConversationHistory()
        elif not isinstance(messages, LLMConversationHistory):
            raise TypeError(
                "Agent.chat expects a LLMConversationHistory instance or None."
            )

        max_retries = 5
        for attempt in range(1, max_retries + 1):
            try:
                return self.conversation_handler.handle_conversation(
                    messages,
                    max_rounds=max_rounds,
                    message_handler=message_handler,
                    verbose_response=runtime_config.get("verbose_response", False),
                    spinner=spinner,
                    max_tokens=max_tokens,
                    verbose_events=runtime_config.get("verbose_events", False),
                    tool_user=tool_user,
                )
            except ProviderError as e:
                error_data = getattr(e, "error_data", {}) or {}
                code = error_data.get("code", "")
                # Retry only on 5xx errors
                if isinstance(code, int) and 500 <= code < 600:
                    pass
                elif (
                    isinstance(code, str) and code.isdigit() and 500 <= int(code) < 600
                ):
                    code = int(code)
                else:
                    raise

                if attempt < max_retries:
                    print(
                        f"ProviderError with 5xx code encountered (attempt {attempt}/{max_retries}). Retrying in 5 seconds..."
                    )
                    time.sleep(5)
                else:
                    print("Max retries reached. Raising error.")
                    raise

                raise



import sys
import subprocess
import tempfile
import time
import http.client
import os
from rich.console import Console
from janito.cli._termweb_log_utils import print_termweb_logs
from janito.i18n import tr


def wait_for_termweb(port, timeout=3.0):
    """Polls the Bottle app root endpoint until it responds or timeout (seconds) is reached."""
    deadline = time.time() + timeout
    while time.time() < deadline:
        try:
            conn = http.client.HTTPConnection("localhost", port, timeout=0.5)
            conn.request("GET", "/")
            resp = conn.getresponse()
            if resp.status == 200:
                return True
        except Exception:
            pass
        time.sleep(0.1)
    return False


def start_termweb(selected_port):
    """
    Start the termweb server on the given port, with rich spinner and logging.
    Returns (termweb_proc, started: bool)
    """
    console = Console()
    with console.status("[cyan]Starting web server...", spinner="dots"):
        # Step 1: Try source path
        app_py_path = os.path.join(os.path.dirname(__file__), "..", "termweb", "app.py")
        app_py_path = os.path.abspath(app_py_path)
        if not os.path.isfile(app_py_path):
            # Step 2: Try installed package
            try:
                import janito_termweb

                app_py_path = janito_termweb.__file__.replace("__init__.py", "app.py")
            except ImportError:
                console.print("[red]Could not find termweb app.py![/red]")
                return None, False, None, None
        termweb_stdout = tempfile.NamedTemporaryFile(
            delete=False, mode="w+", encoding="utf-8"
        )
        termweb_stderr = tempfile.NamedTemporaryFile(
            delete=False, mode="w+", encoding="utf-8"
        )
        termweb_proc = subprocess.Popen(
            [sys.executable, app_py_path, "--port", str(selected_port)],
            stdout=termweb_stdout,
            stderr=termweb_stderr,
        )
        if wait_for_termweb(selected_port, timeout=3.0):
            console.print(
                tr(
                    "TermWeb started... Available at http://localhost:{selected_port}",
                    selected_port=selected_port,
                )
            )
            return termweb_proc, True, termweb_stdout.name, termweb_stderr.name
        else:
            termweb_proc.terminate()
            termweb_proc.wait()
            console.print(
                f"[red]Failed to start TermWeb on port {selected_port}. Check logs for details.[/red]"
            )
            print_termweb_logs(termweb_stdout.name, termweb_stderr.name, console)
            return None, False, termweb_stdout.name, termweb_stderr.name



from janito.agent.tools.run_powershell_command import RunPowerShellCommandTool


def main():
    tool = RunPowerShellCommandTool()
    # Simple PowerShell command
    result = tool.run('Write-Output "Hello from PowerShell"')
    # Write result to file in utf-8 to avoid Windows console encoding issues
    with open("tests/powershell_test_output.txt", "w", encoding="utf-8") as f:
        f.write(result)
    print("✅ Output written to tests/powershell_test_output.txt (UTF-8)")


if __name__ == "__main__":
    main()



from janito.agent.tool_base import ToolBase
from janito.agent.tools_utils.action_type import ActionType
from janito.agent.tool_registry import register_tool
from janito.agent.tools_utils.utils import pluralize, display_path
from janito.i18n import tr
import shutil
import os
import zipfile


@register_tool(name="remove_directory")
class RemoveDirectoryTool(ToolBase):
    """
    Remove a directory.

    Args:
        file_path (str): Path to the directory to remove.
        recursive (bool, optional): If True, remove non-empty directories recursively (with backup). If False, only remove empty directories. Defaults to False.
    Returns:
        str: Status message indicating result. Example:
            - "Directory removed: /path/to/dir"
            - "Error removing directory: <error message>"
    """

    def run(self, file_path: str, recursive: bool = False) -> str:
        disp_path = display_path(file_path)
        self.report_info(
            ActionType.WRITE,
            tr("🗃️ Remove directory '{disp_path}' ...", disp_path=disp_path),
        )
        backup_zip = None
        try:
            if recursive:
                # Backup before recursive removal
                if os.path.exists(file_path) and os.path.isdir(file_path):
                    backup_zip = file_path.rstrip("/\\") + ".bak.zip"
                    with zipfile.ZipFile(backup_zip, "w", zipfile.ZIP_DEFLATED) as zipf:
                        for root, dirs, files in os.walk(file_path):
                            for file in files:
                                abs_path = os.path.join(root, file)
                                rel_path = os.path.relpath(
                                    abs_path, os.path.dirname(file_path)
                                )
                                zipf.write(abs_path, rel_path)
                shutil.rmtree(file_path)
            else:
                os.rmdir(file_path)
            self.report_success(
                tr("✅ 1 {dir_word}", dir_word=pluralize("directory", 1))
            )
            msg = tr("Directory removed: {disp_path}", disp_path=disp_path)
            if backup_zip:
                msg += tr(" (backup at {backup_zip})", backup_zip=backup_zip)
            return msg
        except Exception as e:
            self.report_error(tr(" ❌ Error removing directory: {error}", error=e))
            return tr("Error removing directory: {error}", error=e)



import os
from janito.agent.tool_registry import register_tool

# from janito.agent.tools_utils.expand_path import expand_path
from janito.agent.tools_utils.utils import display_path
from janito.agent.tool_base import ToolBase
from janito.agent.tools_utils.action_type import ActionType
from janito.i18n import tr


from janito.agent.tools.validate_file_syntax.core import validate_file_syntax


@register_tool(name="create_file")
class CreateFileTool(ToolBase):
    """
    Create a new file with the given content.
    Args:
        file_path (str): Path to the file to create.
        content (str): Content to write to the file.
    Returns:
        str: Status message indicating the result. Example:
            - "✅ Successfully created the file at ..."

    Note: Syntax validation is automatically performed after this operation.
    """

    def run(self, file_path: str, content: str) -> str:
        expanded_file_path = file_path  # Using file_path as is
        disp_path = display_path(expanded_file_path)
        file_path = expanded_file_path
        if os.path.exists(file_path):
            try:
                with open(file_path, "r", encoding="utf-8", errors="replace") as f:
                    existing_content = f.read()
            except Exception as e:
                existing_content = f"[Error reading file: {e}]"
            return tr(
                "❗ Cannot create file: file already exists at '{disp_path}'.\n--- Current file content ---\n{existing_content}",
                disp_path=disp_path,
                existing_content=existing_content,
            )
        dir_name = os.path.dirname(file_path)
        if dir_name:
            os.makedirs(dir_name, exist_ok=True)
        self.report_info(
            ActionType.WRITE,
            tr("📝 Create file '{disp_path}' ...", disp_path=disp_path),
        )
        with open(file_path, "w", encoding="utf-8", errors="replace") as f:
            f.write(content)
        new_lines = content.count("\n") + 1 if content else 0
        self.report_success(tr("✅ {new_lines} lines", new_lines=new_lines))
        # Perform syntax validation and append result
        validation_result = validate_file_syntax(file_path)
        return (
            tr("✅ Created file {new_lines} lines.", new_lines=new_lines)
            + f"\n{validation_result}"
        )



"""
Utilities for working with the Rich library.
"""

from rich.console import Console
from typing import Optional


class RichPrinter:
    """
    Utility class for printing styled messages using the Rich library.

    Args:
        console (Optional[Console]): An optional Rich Console instance. If not provided, a default Console will be created.

    Methods:
        print_info(message: str)
            Print an informational message in cyan (no newline at end).

        print_error(message: str)
            Print an error message in bold red.

        print_warning(message: str)
            Print a warning message in bold yellow.

        print_magenta(message: str)
            Print a message in magenta.
    """

    def __init__(self, console: Optional[Console] = None):
        self.console = console or Console()

    def print_info(self, message: str):
        self.console.print(message, style="cyan", end="")

    def print_error(self, message: str):
        self.console.print(message, style="bold red", end="\n")

    def print_warning(self, message: str):
        self.console.print(message, style="bold yellow", end="\n")

    def print_magenta(self, message: str):
        self.console.print(message, style="magenta", end="\n")

    def print_colored_message(self, message: str, color_index: int = 0):
        """
        Print a message with a cycling background color for verbose-messages.
        """
        bg_colors = [
            "on blue",
            "on green",
            "on magenta",
            "on cyan",
            "on yellow",
            "on red",
            "on bright_black",
        ]
        style = f"bold white {bg_colors[color_index % len(bg_colors)]}"
        self.console.print(message, style=style, end="\n")



# Centralized config defaults for Janito
CONFIG_DEFAULTS = {
    "api_key": None,  # Must be set by user
    "model": "gpt-4.1",  # Default model
    "role": "software developer",  # Part of the Agent Profile
    "system_prompt_template": None,  # None means auto-generate from Agent Profile role
    "temperature": 0.2,
    "max_tokens": 32000,
    "use_azure_openai": False,
    "azure_openai_api_version": "2023-05-15",
    "profile": "base",
}



from janito.agent.tool_base import ToolBase
from janito.agent.tools_utils.action_type import ActionType
from janito.agent.tool_registry import register_tool
from janito.agent.tools_utils.utils import pluralize, display_path
from janito.agent.tools_utils.dir_walk_utils import walk_dir_with_gitignore
from janito.i18n import tr
import fnmatch
import os


@register_tool(name="find_files")
class FindFilesTool(ToolBase):
    """
    Find files or directories in one or more directories matching a pattern. Respects .gitignore.
    Args:
        paths (str): String of one or more paths (space-separated) to search in. Each path can be a directory.
        pattern (str): File pattern(s) to match. Multiple patterns can be separated by spaces. Uses Unix shell-style wildcards (fnmatch), e.g. '*.py', 'data_??.csv', '[a-z]*.txt'.
            - If the pattern ends with '/' or '\\', only matching directory names (with trailing slash) are returned, not the files within those directories. For example, pattern '*/' will return only directories at the specified depth.
        max_depth (int, optional): Maximum directory depth to search. If None, unlimited recursion. If 0, only the top-level directory. If 1, only the root directory (matches 'find . -maxdepth 1').
        max_results (int, optional): Maximum number of results to return. 0 means no limit (default).
    Returns:
        str: Newline-separated list of matching file paths. Example:
            "/path/to/file1.py\n/path/to/file2.py"
            "Warning: Empty file pattern provided. Operation skipped."
            If max_results is reached, appends a note to the output.
    """

    def _match_directories(self, root, dirs, pat):
        dir_output = set()
        dir_pat = pat.rstrip("/\\")
        for d in dirs:
            if fnmatch.fnmatch(d, dir_pat):
                dir_output.add(os.path.join(root, d) + os.sep)
        return dir_output

    def _match_files(self, root, files, pat):
        file_output = set()
        for filename in fnmatch.filter(files, pat):
            file_output.add(os.path.join(root, filename))
        return file_output

    def _match_dirs_without_slash(self, root, dirs, pat):
        dir_output = set()
        for d in fnmatch.filter(dirs, pat):
            dir_output.add(os.path.join(root, d))
        return dir_output

    def run(self, paths: str, pattern: str, max_depth: int = None) -> str:
        if not pattern:
            self.report_warning(tr("ℹ️ Empty file pattern provided."))
            return tr("Warning: Empty file pattern provided. Operation skipped.")
        patterns = pattern.split()
        results = []
        for directory in paths.split():
            disp_path = display_path(directory)
            depth_msg = (
                tr(" (max depth: {max_depth})", max_depth=max_depth)
                if max_depth is not None and max_depth > 0
                else ""
            )
            self.report_info(
                ActionType.READ,
                tr(
                    "🔍 Search for files '{pattern}' in '{disp_path}'{depth_msg} ...",
                    pattern=pattern,
                    disp_path=disp_path,
                    depth_msg=depth_msg,
                ),
            )
            dir_output = set()
            for root, dirs, files in walk_dir_with_gitignore(
                directory, max_depth=max_depth
            ):
                for pat in patterns:
                    if pat.endswith("/") or pat.endswith("\\"):
                        dir_output.update(self._match_directories(root, dirs, pat))
                    else:
                        dir_output.update(self._match_files(root, files, pat))
                        dir_output.update(
                            self._match_dirs_without_slash(root, dirs, pat)
                        )
            self.report_success(
                tr(
                    " ✅ {count} {file_word}",
                    count=len(dir_output),
                    file_word=pluralize("file", len(dir_output)),
                )
            )
            if directory.strip() == ".":
                dir_output = {
                    p[2:] if (p.startswith("./") or p.startswith(".\\")) else p
                    for p in dir_output
                }
            results.extend(sorted(dir_output))
        result = "\n".join(results)
        return result



import os
import urllib.parse
from janito.agent.runtime_config import runtime_config


def display_path(path):
    """
    Returns a display-friendly path. If runtime_config['termweb_port'] is set, injects an ANSI hyperlink to the local web file viewer.
    Args:
        path (str): Path to display.
    Returns:
        str: Display path, optionally as an ANSI hyperlink.
    """
    if os.path.isabs(path):
        cwd = os.path.abspath(os.getcwd())
        abs_path = os.path.abspath(path)
        # Check if the absolute path is within the current working directory
        if abs_path.startswith(cwd + os.sep):
            disp = os.path.relpath(abs_path, cwd)
        else:
            disp = path
    else:
        disp = os.path.relpath(path)
    port = runtime_config.get("termweb_port")
    if port:
        url = f"http://localhost:{port}/?path={urllib.parse.quote(path)}"
        # Use Rich markup for hyperlinks
        return f"[link={url}]{disp}[/link]"
    return disp


def pluralize(word: str, count: int) -> str:
    """Return the pluralized form of word if count != 1, unless word already ends with 's'."""
    if count == 1 or word.endswith("s"):
        return word
    return word + "s"



import http.client
from rich.console import Console
from janito.agent.runtime_config import runtime_config


def is_termweb_running(port):
    """Check if termweb is running by making an HTTP request to the root endpoint."""
    try:
        conn = http.client.HTTPConnection("localhost", port, timeout=0.5)
        conn.request("GET", "/")
        resp = conn.getresponse()
        return resp.status == 200
    except Exception:
        return False


def handle_termweb_log_tail(console: Console, *args, shell_state=None, **kwargs):
    lines = 20
    if args and args[0].isdigit():
        lines = int(args[0])
    stdout_path = shell_state.termweb_stdout_path if shell_state else None
    stderr_path = shell_state.termweb_stderr_path if shell_state else None
    if not stdout_path and not stderr_path:
        console.print(
            "[yellow][termweb] No termweb log files found for this session.[/yellow]"
        )
        return
    if stdout_path:
        try:
            with open(stdout_path, encoding="utf-8") as f:
                stdout_lines = f.readlines()[-lines:]
            if stdout_lines:
                console.print(
                    f"[yellow][termweb][stdout] Tail of {stdout_path}:\n"
                    + "".join(stdout_lines)
                )
        except Exception:
            pass
    if stderr_path:
        try:
            with open(stderr_path, encoding="utf-8") as f:
                stderr_lines = f.readlines()[-lines:]
            if stderr_lines:
                console.print(
                    f"[red][termweb][stderr] Tail of {stderr_path}:\n"
                    + "".join(stderr_lines)
                )
        except Exception:
            pass
    if (not stdout_path or not stdout_lines) and (not stderr_path or not stderr_lines):
        console.print("[termweb] No output or errors captured in logs.")


handle_termweb_log_tail.help_text = "Show the last lines of the latest termweb logs"


def handle_termweb_status(console: Console, *args, shell_state=None, **kwargs):
    if shell_state is None:
        console.print(
            "[red]No shell state available. Cannot determine termweb status.[/red]"
        )
        return
    port = getattr(shell_state, "termweb_port", None)
    port_source = "shell_state"
    if not port:
        port = runtime_config.get("termweb_port")
        port_source = "runtime_config"
    pid = getattr(shell_state, "termweb_pid", None)
    stdout_path = getattr(shell_state, "termweb_stdout_path", None)
    stderr_path = getattr(shell_state, "termweb_stderr_path", None)
    running = False
    if port:
        running = is_termweb_running(port)
    console.print("[bold cyan]TermWeb Server Status:[/bold cyan]")
    console.print(f"  Running: {'[green]Yes[/green]' if running else '[red]No[/red]'}")
    if pid:
        console.print(f"  PID: {pid}")
    if port:
        console.print(f"  Port: {port} (from {port_source})")
        url = f"http://localhost:{port}/"
        console.print(f"  URL: [underline blue]{url}[/underline blue]")
    else:
        console.print(
            "  [yellow]No port configured in state or runtime_config.[/yellow]"
        )
    if stdout_path:
        console.print(f"  Stdout log: {stdout_path}")
    if stderr_path:
        console.print(f"  Stderr log: {stderr_path}")


handle_termweb_status.help_text = (
    "Show status information about the running termweb server"
)



import sys
import os

sys.path.insert(0, os.path.abspath("."))
from janito.agent.tools.get_file_outline.python_outline import parse_python_outline


def test_parse_python_outline_accuracy():
    code = [
        "class A:",  # 1
        "    def foo(self):",  # 2
        "        pass",  # 3
        "",  # 4
        "    def bar(self):",  # 5
        "        pass",  # 6
        "",  # 7
        "def top():",  # 8
        "    pass",  # 9
        "",  # 10
        "class B:",  # 11
        "    pass",  # 12
    ]
    outline = parse_python_outline(code)
    print("OUTLINE OUTPUT:")
    for item in outline:
        print(item)

    # Find by name/type for clarity
    def find(type_, name):
        return next(x for x in outline if x["type"] == type_ and x["name"] == name)

    a = find("class", "A")
    foo = find("method", "foo")
    bar = find("method", "bar")
    top = find("function", "top")
    b = find("class", "B")
    assert a["start"] == 1 and a["end"] == 7, f"A: {a}"
    assert foo["start"] == 2 and foo["end"] == 3, f"foo: {foo}"
    assert bar["start"] == 5 and bar["end"] == 6, f"bar: {bar}"
    assert top["start"] == 8 and top["end"] == 10, f"top: {top}"
    assert b["start"] == 11 and b["end"] == 12, f"B: {b}"
    print("parse_python_outline accuracy test passed.")


if __name__ == "__main__":
    test_parse_python_outline_accuracy()



import pytest
from janito.agent.tools_utils.formatting import OutlineFormatter


def test_format_outline_table_empty():
    assert (
        OutlineFormatter.format_outline_table([])
        == "No classes, functions, or variables found."
    )


def test_format_outline_table_basic():
    items = [
        {
            "type": "class",
            "name": "Foo",
            "start": 1,
            "end": 10,
            "parent": "",
            "docstring": "A class.",
        },
        {
            "type": "function",
            "name": "bar",
            "start": 12,
            "end": 15,
            "parent": "Foo",
            "docstring": "Does bar.",
        },
    ]
    result = OutlineFormatter.format_outline_table(items)
    assert "| class   | Foo" in result
    assert "| function | bar" in result
    assert "A class." in result
    assert "Does bar." in result


def test_format_markdown_outline_table_empty():
    assert OutlineFormatter.format_markdown_outline_table([]) == "No headers found."


def test_format_markdown_outline_table_basic():
    items = [
        {"level": 1, "title": "Header 1", "line": 1},
        {"level": 2, "title": "Header 2", "line": 3},
    ]
    result = OutlineFormatter.format_markdown_outline_table(items)
    assert "| 1     | Header 1" in result
    assert "| 2     | Header 2" in result



"""Main CLI entry point for Janito."""

from janito.cli.arg_parser import create_parser
from janito.cli.config_commands import handle_config_commands
from janito.cli.logging_setup import setup_verbose_logging
from janito.cli.cli_main import run_cli
from janito.agent.runtime_config import unified_config
from janito.cli.livereload_starter import start_livereload

# Ensure all tools are registered at startup
import janito.agent.tools  # noqa: F401
from janito.i18n import tr


def handle_list_sessions(args):
    import os
    import glob

    n = args.list if args.list is not None else 10
    history_dir = os.path.join(os.path.expanduser(".janito"), "chat_history")
    if not os.path.exists(history_dir):
        print("No session history found.")
        return True
    files = glob.glob(os.path.join(history_dir, "*.json"))
    files = sorted(files, key=os.path.getmtime, reverse=True)
    print(f"Last {n} sessions:")
    for f in files[:n]:
        session_id = os.path.splitext(os.path.basename(f))[0]
        print(session_id)
    return True


def handle_view_session(args):
    import os
    import json

    history_dir = os.path.join(os.path.expanduser(".janito"), "chat_history")
    session_file = os.path.join(history_dir, f"{args.view}.json")
    if not os.path.exists(session_file):
        print(f"Session '{args.view}' not found.")
        return 1
    with open(session_file, "r", encoding="utf-8") as f:
        try:
            messages = json.load(f)
        except Exception as e:
            print(f"Failed to load session: {e}")
            return 1
    print(f"Conversation history for session '{args.view}':\n")
    for i, msg in enumerate(messages, 1):
        role = msg.get("role", "?")
        content = msg.get("content", "")
        print(f"[{i}] {role}: {content}\n")
    return 0


def handle_lang_selection(args):
    import janito.i18n as i18n
    from janito.agent.runtime_config import runtime_config

    lang = getattr(args, "lang", None) or unified_config.get("lang", None) or "en"
    runtime_config.set("lang", lang)
    i18n.set_locale(lang)


def handle_help_config(args):
    from janito.agent.config import CONFIG_OPTIONS
    from janito.agent.config_defaults import CONFIG_DEFAULTS

    print(tr("Available configuration options:\n"))
    for key, desc in CONFIG_OPTIONS.items():
        default = CONFIG_DEFAULTS.get(key, None)
        print(
            tr(
                "{key:15} {desc} (default: {default})",
                key=key,
                desc=desc,
                default=default,
            )
        )


def handle_list_tools(args):
    from janito.agent.tool_registry import get_tool_schemas
    from rich.console import Console
    from rich.table import Table

    console = Console()
    table = Table(
        title="Ferramentas Registradas", show_lines=True, style="bold magenta"
    )
    table.add_column("Gnome", style="cyan", no_wrap=True)
    table.add_column("Descrição", style="green")
    table.add_column("Parâmetros", style="yellow")
    for schema in get_tool_schemas():
        fn = schema["function"]
        params = "\n".join(
            [
                f"[bold]{k}[/]: {v['type']}"
                for k, v in fn["parameters"].get("properties", {}).items()
            ]
        )
        table.add_row(f"[b]{fn['name']}[/b]", fn["description"], params or "-")
    console.print(table)


def handle_info(args):
    from janito import __version__
    from rich.console import Console
    from rich.text import Text
    from janito.agent.runtime_config import runtime_config

    model = unified_config.get("model")
    temperature = unified_config.get("temperature")
    max_tokens = unified_config.get("max_tokens")
    system_prompt_val = None
    if getattr(args, "system_file", None):
        try:
            with open(args.system_file, "r", encoding="utf-8") as f:
                system_prompt_val = f.read().strip()
            runtime_config.set("system_prompt_template", system_prompt_val)
        except Exception as e:
            system_prompt_val = f"(error reading system-file: {e})"
    elif getattr(args, "system", None):
        system_prompt_val = args.system
        runtime_config.set("system_prompt_template", system_prompt_val)
    else:
        system_prompt_val = runtime_config.get("system_prompt_template")
    if not system_prompt_val:
        try:
            from janito.agent.profile_manager import AgentProfileManager
            from janito.agent.config import get_api_key

            role = args.role or unified_config.get("role", "software engineer")
            interaction_mode = "chat" if not getattr(args, "prompt", None) else "prompt"
            profile = "base"
            profile_manager = AgentProfileManager(
                api_key=get_api_key(),
                model=unified_config.get("model"),
            )
            system_prompt_val = profile_manager.get_system_prompt(
                role, interaction_mode, profile
            )
        except Exception as e:
            system_prompt_val = f"(error: {e})"
    console = Console()
    info_text = Text()
    info_text.append(f"Janito v{__version__}", style="bold cyan")
    info_text.append(" | model: ", style="white")
    info_text.append(str(model), style="green")
    info_text.append(" | temp: ", style="white")
    info_text.append(str(temperature), style="yellow")
    info_text.append(" | max_tokens: ", style="white")
    info_text.append(str(max_tokens), style="magenta")
    info_text.append(" | system: ", style="white")
    info_text.append(str(system_prompt_val), style="bold blue")
    console.print(info_text, style="dim")


def main():
    """Unified entry point for the Janito CLI and web server."""
    import sys
    from janito.agent.config import local_config, global_config

    local_config.load()
    global_config.load()
    parser = create_parser()
    args = parser.parse_args()
    # Handle --list [n] before anything else
    if getattr(args, "list", None) is not None:
        if handle_list_sessions(args):
            sys.exit(0)
    # Handle --view <id> to print conversation history
    if getattr(args, "view", None) is not None:
        sys.exit(handle_view_session(args))
    handle_lang_selection(args)
    if getattr(args, "help_config", False):
        handle_help_config(args)
        sys.exit(0)
    if getattr(args, "list_tools", False):
        handle_list_tools(args)
        sys.exit(0)
    if getattr(args, "info", False):
        handle_info(args)
    handle_config_commands(args)
    setup_verbose_logging(args)
    if getattr(args, "web", False):
        import subprocess

        subprocess.run([sys.executable, "-m", "janito.web"])
    elif getattr(args, "live", False):
        port = 35729
        livereload_proc, started, livereload_stdout_path, livereload_stderr_path = (
            start_livereload(port)
        )
        try:
            run_cli(args)
        finally:
            if livereload_proc:
                livereload_proc.terminate()
                livereload_proc.wait()
    else:
        run_cli(args)



from janito.agent.message_handler_protocol import MessageHandlerProtocol


class QueueMessageHandler(MessageHandlerProtocol):
    def __init__(self, queue, *args, **kwargs):
        self._queue = queue

    def handle_tool_call(self, tool_call):
        # All output is routed through the unified message handler and queue
        return super().handle_tool_call(tool_call)

    def handle_message(self, msg, msg_type=None):
        # Unified: send content (agent/LLM) messages to the frontend
        if not isinstance(msg, dict):
            raise TypeError(
                f"QueueMessageHandler.handle_message expects a dict with 'type' and 'message', got {type(msg)}: {msg!r}"
            )
        msg_type = msg.get("type", "info")
        message = msg.get("message", "")
        self._queue.put(("message", message, msg_type))
